UNIT electronic;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Gets electronic contribution to thermodynamic properties including the
   contribution to pressure. See manual chapter 12.
   Electronic contributions can be described as:
   (1). Free electron gas
   (2). Extended free electron gas: parameterization of phonon-electron coupling as in 
        platinum
   (3). Crystal Field with M1, M2 sites as in fayalite Fe2SiO4
   In (1) and (2) the same formalism is used; in the normal free electron gas model (1)
   all coefficients are zero except for C_EL[1] and GAMMA_EL.
   In (3) all energy levels may have a different gruneisen parameter which is defined
   as gamma_cf[i]=-dLn(E_CF[i])/dLn(V) at constant temperature; if these are zero the
   crystal field contribution to total pressure is zero
 *)

INTERFACE
USES CRT,sysutils,variables;


PROCEDURE ELECTRONIC_PROPERTIES(VPT:EXTENDED;VAR A_ELEC,U_ELEC,S_ELEC,CV_ELEC,
                                K_ELEC,DPDT_ELEC:EXTENDED);
FUNCTION P_ELECTRON(VPT:EXTENDED):EXTENDED;
IMPLEMENTATION


PROCEDURE GET_ELECTRONIC(VPT:EXTENDED;WHAT:INTEGER;VAR PROP:EXTENDED);
(* Returns an electronic property, depending on WHAT.
   WHAT=1: Cv, WHAT=2: entropy, WHAT=3: Helmholtz energy. From these 
   properties all other thermodynamic contributions can be calculated.
   See me-freepascal-14sep2017, section 12,13 and 14. The free-electron gas
   model can also be treated by the extended free-electron gas model
 *)
VAR DELTA,ETA,EXP_B1,EXP_B2,EXP_B3,SUM,VOLUME_FACTOR,Z1,Z2:EXTENDED;
    I:INTEGER;
BEGIN
  ETA:=VPT/V0;
  VOLUME_FACTOR:=POW_N(ETA,GAMMA_EL);
  DELTA:=1E-12; PROP:=0;
  IF WHAT=1 THEN
  BEGIN
    (* Isochoric heat capacity, eqn. (13.1) *)
    SUM:=0;
    FOR I:=1 TO 3 DO
    BEGIN
      SUM:=SUM+A_EL[I]*INT_POW_N(TEMPERATURE,I)*EXP(-B_EL[I]*TEMPERATURE)
    END;
    SUM:=SUM+C_EL[1]*TEMPERATURE;
    SUM:=SUM+C_EL[2]*(1-1/(C_EL[3]*TEMPERATURE+1));
    PROP:=SUM*VOLUME_FACTOR
  END;
  IF WHAT=2 THEN
  BEGIN
    (* Entropy, eqn. (13.7) *)
    EXP_B1:=EXP(-B_EL[1]*TEMPERATURE);
    EXP_B2:=EXP(-B_EL[2]*TEMPERATURE);
    EXP_B3:=EXP(-B_EL[3]*TEMPERATURE);
    IF ABS(B_EL[1])>DELTA THEN
    BEGIN
      SUM:=A_EL[1]*(1-EXP_B1)/B_EL[1]
    END
    ELSE SUM:=A_EL[1]*TEMPERATURE;
    IF ABS(B_EL[2])>DELTA THEN
    BEGIN
      SUM:=SUM-A_EL[2]*TEMPERATURE*EXP_B2/B_EL[2];
      SUM:=SUM+A_EL[2]*(1-EXP_B2)/B_EL[2]/B_EL[2]
    END
    ELSE SUM:=SUM+0.5*A_EL[2]*TEMPERATURE*TEMPERATURE;
    IF ABS(B_EL[3])>DELTA THEN
    BEGIN
      SUM:=SUM-A_EL[3]*TEMPERATURE*TEMPERATURE*EXP_B3/B_EL[3];
      Z1:=-TEMPERATURE*EXP_B3/B_EL[3];
      Z1:=Z1+(1-EXP_B3)/B_EL[3]/B_EL[3];
      SUM:=SUM+2*A_EL[3]*Z1/B_EL[3]
    END
    ELSE SUM:=SUM+A_EL[3]*INT_POW_N(TEMPERATURE,3)/3;
    SUM:=SUM+C_EL[1]*TEMPERATURE;
    SUM:=SUM+C_EL[2]*LN(C_EL[3]*TEMPERATURE+1);
    PROP:=SUM*VOLUME_FACTOR
  END;
  IF WHAT=3 THEN
  BEGIN
    (* Helmholtz energy, eqn. (13.8) *)
    EXP_B1:=EXP(-B_EL[1]*TEMPERATURE);
    EXP_B2:=EXP(-B_EL[2]*TEMPERATURE);
    EXP_B3:=EXP(-B_EL[3]*TEMPERATURE);
    SUM:=0;
    IF ABS(B_EL[1])>DELTA THEN
    BEGIN
      SUM:=-A_EL[1]*TEMPERATURE/B_EL[1];
      SUM:=SUM+A_EL[1]*(1-EXP_B1)/B_EL[1]/B_EL[1];
    END
    ELSE SUM:=-0.5*A_EL[1]*TEMPERATURE*TEMPERATURE;
    IF ABS(B_EL[2])>DELTA THEN
    BEGIN
      SUM:=SUM-A_EL[2]*TEMPERATURE*(1+EXP_B2)/B_EL[2]/B_EL[2];
      SUM:=SUM+2*A_EL[2]*(1-EXP_B2)/INT_POW_N(B_EL[2],3);
    END
    ELSE SUM:=SUM-A_EL[2]*INT_POW_N(TEMPERATURE,3)/6;
    IF ABS(B_EL[3])>DELTA THEN
    BEGIN
      Z1:=-TEMPERATURE*TEMPERATURE*EXP_B3/B_EL[3];
      Z2:=-TEMPERATURE*EXP_B3+(1-EXP_B3)/B_EL[3];
      Z2:=2*Z2/B_EL[3]/B_EL[3];
      SUM:=SUM+A_EL[3]*(Z1+Z2)/B_EL[3];
      SUM:=SUM-2*A_EL[3]*TEMPERATURE*(EXP_B3+1)/INT_POW_N(B_EL[3],3)
    END
    ELSE SUM:=SUM-A_EL[3]*INT_POW_N(TEMPERATURE,4)/12;
    SUM:=SUM-0.5*C_EL[1]*TEMPERATURE*TEMPERATURE;
    IF ABS(C_EL[3])>DELTA THEN
    BEGIN
      Z1:=(C_EL[3]*TEMPERATURE+1)*LN(C_EL[3]*TEMPERATURE+1);
      Z1:=Z1-C_EL[3]*TEMPERATURE;
      SUM:=SUM-C_EL[2]*Z1/C_EL[3]
    END;
    PROP:=SUM*VOLUME_FACTOR
  END
END;

PROCEDURE GET_GRUN_EJ(VPT,GAMMA:EXTENDED;VAR EJ:EXTENDED);
(* The electronic gruneisen parameter is used to calculate the energy of a level
   because the energy levels may be volume dependent. The incoming energy EJ
   is that at 0 K and 0 GPa. See section 14, eqn. (14.9)
 *)
VAR Z:EXTENDED;
BEGIN
  IF ABS(GAMMA)>1E-8 THEN
  BEGIN
    Z:=-GAMMA*LN(VPT/V0);
    EJ:=EJ*EXP(Z)
  END
END;

PROCEDURE GET_CRYSTAL_FIELD(VPT:EXTENDED;VAR A,U,S,CV,DPDT,KBULK:EXTENDED);
(* Calculates the electronic properties for a substances having two sites.
   me-freepascal-14sep2017.doc. When groundstate is degenerate (GJ>1) then a
   correction for entropy and Helmholtz is required and other properties do
   not change.
   A=Helmholtz energy, U=Energy, S=Entropy, CV=isochoric heat capacity,
   DPDT=isochoric temperature derivative of electronic pressure, KBULK=isothermal
   bulk modulus contribution. See me-freepascal-14sep2017.doc, section 14
 *)
VAR CONVERSION_HCK,EJ,GAMMA,GJ,G1,DUMMY,XJ,Z_A,Z_S,Z_CV,Z_P,Z_K,Z_DPDT,Z_EXP,
    A_ELEC,U_ELEC,S_ELEC,CV_ELEC,K_ELEC,DPDT_ELEC:EXTENDED;
    I,J,NLEV:INTEGER;
BEGIN
  A:=0; U:=0; S:=0; CV:=0; DPDT:=0; KBULK:=0;
  (* Conversion factor for calculating wavenumbers (cm^-1) to energy (Joules) *)
  CONVERSION_HCK:=100*PLANCK*LIGHT_SPEED/BOLTZMANN;
  (* At zero temperature all properties are zero *)
  IF TEMPERATURE>0.01 THEN
  BEGIN
    (* Maximum 3 sites M1, M2, M3 denoted by I *)
    FOR I:=1 TO NUM_CF_SITES DO
    BEGIN
      Z_A:=0; Z_S:=0; Z_CV:=0; Z_P:=0; Z_K:=0; Z_DPDT:=0;
      (* Site M1 has M1_LEVELS, M2 has M2_LEVELS and M3 has M3_levels energy levels *)
      CASE I OF
        1:NLEV:=M1_LEVELS;
        2:NLEV:=M2_LEVELS;
        3:NLEV:=M3_LEVELS
      END;
      FOR J:=1 TO NLEV DO
      BEGIN
        CASE I OF
          1:BEGIN
              EJ:=E_CF[J]; GJ:=G_CF[J];
              G1:=G_CF[1]; GAMMA:=GAMMA_CF[J]
            END;
          2:BEGIN
              EJ:=E_CF[M1_LEVELS+J]; GJ:=G_CF[M1_LEVELS+J];
              G1:=G_CF[M1_LEVELS+1]; GAMMA:=GAMMA_CF[M1_LEVELS+J]
            END;
          3:BEGIN
              EJ:=E_CF[M1_LEVELS+M2_LEVELS+J]; GJ:=G_CF[M1_LEVELS+M2_LEVELS+J];
              G1:=G_CF[M1_LEVELS+M2_LEVELS+1]; GAMMA:=GAMMA_CF[M1_LEVELS+M2_LEVELS+J]
            END
        END;
        (* Determine energy: it may be volume dependent when gamma<>0 *)
        GET_GRUN_EJ(VPT,GAMMA,EJ);
        (* Xj, section 14, eqn. (14.6b) converted from wavenumbers, cm^-1:
           100*h*c*wavenumber/(k*T)
         *)
        XJ:=CONVERSION_HCK*EJ/TEMPERATURE;
        Z_EXP:=-XJ;
        IF Z_EXP<-MAX_ARG_EXP THEN Z_EXP:=0 ELSE Z_EXP:=EXP(Z_EXP);
        (* Summations in section 14, eqn. (14.16) *)
        Z_A:=Z_A+GJ*Z_EXP;
        Z_S:=Z_S+GJ*XJ*Z_EXP;
        Z_CV:=Z_CV+GJ*XJ*XJ*Z_EXP;
        Z_P:=Z_P+GJ*GAMMA*XJ*Z_EXP;
        Z_K:=Z_K+GJ*GAMMA*GAMMA*(XJ*XJ-XJ)*Z_EXP;
        Z_DPDT:=Z_DPDT+GJ*GAMMA*XJ*XJ*Z_EXP
      END;
      IF NLEV>0 THEN
      BEGIN
        (* Helmholtz energy, section 14, eqn. (14.38) *)
        A:=A-NUM_CF_ATOMS*GAS_CONSTANT*TEMPERATURE*LN(Z_A/G1);
        (* Energy, section 14, eqn. (14.19) *)
        U:=U+NUM_CF_ATOMS*GAS_CONSTANT*TEMPERATURE*Z_S/Z_A;
        (* Entropy, section 14, eqn. (14.40) *)
        S:=S+NUM_CF_ATOMS*GAS_CONSTANT*(LN(Z_A/G1)+Z_S/Z_A);
        (* Isochoric heat capacity Cv, section 14, eqn. (14.20) *)
        DUMMY:=Z_S/Z_A;
        CV:=CV+NUM_CF_ATOMS*GAS_CONSTANT*(Z_CV/Z_A-DUMMY*DUMMY);
        (* Bulk modulus K, section 14, eqn. (14.22) *)
        DUMMY:=Z_P/Z_A;
        KBULK:=KBULK+NUM_CF_ATOMS*GAS_CONSTANT*TEMPERATURE*(DUMMY-(Z_K/Z_A-DUMMY*DUMMY))/VPT;
        (* dP/dT at constant volume, section 14, eqn. (14.23) *)
        DPDT:=DPDT+NUM_CF_ATOMS*GAS_CONSTANT*(Z_DPDT/Z_A-Z_P*Z_S/Z_A/Z_A)/VPT
      END
    END;
    (* I assume that there is an equal number of M1,M2,M3 sites:
       e.g 1/2 mol M1 and 1/2 mol M2 when there are 2 sites. Therefore each
       contribution in the I-loop should be counted 1/NUM_CF_SITES.
       See section 14.4, eqn. (14.42)
     *)
    A:=A/NUM_CF_SITES; U:=U/NUM_CF_SITES; S:=S/NUM_CF_SITES; CV:=CV/NUM_CF_SITES;
    KBULK:=KBULK/NUM_CF_SITES; DPDT:=DPDT/NUM_CF_SITES;
    (* Additional contribution Free-electron gas model.
       The electronic coefficient is defined per mole of atoms that give crystal-field
       splitting: in Fe2SiO4 it is defined per mole of Fe
       Helmholtz energy
     *)
    A_ELEC:=0; GET_ELECTRONIC(VPT,3,A_ELEC); A:=A+NUM_CF_ATOMS*A_ELEC;
    (* Entropy *)
    S_ELEC:=0; GET_ELECTRONIC(VPT,2,S_ELEC); S:=S+NUM_CF_ATOMS*S_ELEC;
    (* Energy *)
    U_ELEC:=A_ELEC+TEMPERATURE*S_ELEC; U:=U+NUM_CF_ATOMS*U_ELEC;
    (* Isochoric heat capacity *)
    CV_ELEC:=0; GET_ELECTRONIC(VPT,1,CV_ELEC); CV:=CV+NUM_CF_ATOMS*CV_ELEC;
    (* Isothermal bulk modulus, eqn. (13.10) *)
    K_ELEC:=GAMMA_EL*(GAMMA_EL-1)*A_ELEC/VPT; KBULK:=KBULK+NUM_CF_ATOMS*K_ELEC;
    (* Isochoric temperature dervative of electronic pressure, eqn. (13.11) *)
    DPDT_ELEC:=GAMMA_EL*S_ELEC/VPT; DPDT:=DPDT+NUM_CF_ATOMS*DPDT_ELEC
  END
END;

PROCEDURE ELECTRONIC_PROPERTIES(VPT:EXTENDED;VAR A_ELEC,U_ELEC,S_ELEC,CV_ELEC,
                                K_ELEC,DPDT_ELEC:EXTENDED);
(* Calculates electronic contributions to Helmholtz energy, energy, entropy,
   isochoric heat capacity, isothermal bulk modulus, isochoric temperature
   derivative of pressure. See me-freepascal-14sep2017.doc, section 12,13 and 14
 *)
BEGIN
  (* Electronic_yes=1,2: free electron gas or extended free electron gas *)
  IF ELECTRONIC_YES<3 THEN
  BEGIN
    (* Helmholtz energy *)
    A_ELEC:=0; GET_ELECTRONIC(VPT,3,A_ELEC);
    (* Entropy *)
    S_ELEC:=0; GET_ELECTRONIC(VPT,2,S_ELEC);
    (* Energy *)
    U_ELEC:=A_ELEC+TEMPERATURE*S_ELEC;
    (* Isochoric heat capacity *)
    CV_ELEC:=0; GET_ELECTRONIC(VPT,1,CV_ELEC);
    (* Isothermal bulk modulus, eqn. (13.10) *)
    K_ELEC:=GAMMA_EL*(GAMMA_EL-1)*A_ELEC/VPT;
    (* Isochoric temperature dervative of electronic pressure, eqn. (13.11) *)
    DPDT_ELEC:=GAMMA_EL*S_ELEC/VPT
  END
  ELSE
  BEGIN
    (* Crystal Field contribution, section 14
       Helmholtz energy=A_ELEC; Energy=U_ELEC; Entropy=S_ELEC;
       Isochoric heat capacity=CV_ELEC; Bulk modulus=K_ELEC;
       Isochoric temperature derivative of crystal field pressure=DPDT_ELEC
     *)
    A_ELEC:=0; U_ELEC:=0; S_ELEC:=0; CV_ELEC:=0; DPDT_ELEC:=0; K_ELEC:=0;
    GET_CRYSTAL_FIELD(VPT,A_ELEC,U_ELEC,S_ELEC,CV_ELEC,DPDT_ELEC,K_ELEC)
  END
END;

PROCEDURE P_CRYSTAL_FIELD(VPT:EXTENDED;VAR P_EL:EXTENDED);
(* Calculates the electronic pressure for one endmember having two sites.
   me-freepascal-15jul2016.doc, section 14, eqn. (14.21)
 *)
VAR CONVERSION_HCK,EJ,GAMMA,GJ,G1,XJ,Z_A,Z_P,Z_EXP,A_ELEC:EXTENDED;
    I,J,NLEV:INTEGER;
BEGIN
  P_EL:=0;
  (* At zero temperature all properties are zero *)
  IF TEMPERATURE>0.01 THEN
  BEGIN
    (* Conversion factor for calculating wavenumbers (cm^-1) to energy (Joules) *)
    CONVERSION_HCK:=100*PLANCK*LIGHT_SPEED/BOLTZMANN;
    (* Maximum of 3 sites M1, M2, M3 denoted by I *)
    FOR I:=1 TO NUM_CF_SITES DO
    BEGIN
      Z_A:=0; Z_P:=0;
      (* Site M1 has M1_LEVELS, M2 has M2_LEVELS and M3 has M3_levels energy levels *)
      CASE I OF
        1:NLEV:=M1_LEVELS;
        2:NLEV:=M2_LEVELS;
        3:NLEV:=M3_LEVELS
      END;
      FOR J:=1 TO NLEV DO
      BEGIN
        CASE I OF
          1:BEGIN
              EJ:=E_CF[J]; GJ:=G_CF[J];
              G1:=G_CF[1]; GAMMA:=GAMMA_CF[J]
            END;
          2:BEGIN
              EJ:=E_CF[M1_LEVELS+J]; GJ:=G_CF[M1_LEVELS+J];
              G1:=G_CF[M1_LEVELS+1]; GAMMA:=GAMMA_CF[M1_LEVELS+J]
            END;
          3:BEGIN
              EJ:=E_CF[M1_LEVELS+M2_LEVELS+J]; GJ:=G_CF[M1_LEVELS+M2_LEVELS+J];
              G1:=G_CF[M1_LEVELS+M2_LEVELS+1]; GAMMA:=GAMMA_CF[M1_LEVELS+M2_LEVELS+J]
            END
        END;
        (* Determine energy: it may be volume dependent when gamma<>0 *)
        GET_GRUN_EJ(VPT,GAMMA,EJ);
        (* Xj, section 14, eqn. (14.6b) converted from wavenumbers, cm^-1:
           Xj=100*h*c*wavenumber/(k*T)
         *)
        XJ:=CONVERSION_HCK*EJ/TEMPERATURE;
        Z_EXP:=-XJ;
        IF Z_EXP<-MAX_ARG_EXP THEN Z_EXP:=0 ELSE Z_EXP:=EXP(Z_EXP);
        (* Summations in section 14, eqn. (14.16 *)
        Z_A:=Z_A+GJ*Z_EXP;
        Z_P:=Z_P+GJ*GAMMA*XJ*Z_EXP
      END;
      IF NLEV>0 THEN
      BEGIN
        (* Electronic pressure P_EL, section 14, eqn. (14.21) *)
        P_EL:=P_EL+NUM_CF_ATOMS*GAS_CONSTANT*TEMPERATURE*Z_P/Z_A/VPT
      END
    END;
    (* I assume that the number of M1,M2,M3 sites are equal: for example for
       2 sites M1,M2, 1/2 mol M1 and 1/2 mol M2. Therefore each contribution in
       the I-loop should be counted 1/NUM_CF_SITES. See section 14.4, eqn. (14.42)
     *)
    P_EL:=P_EL/NUM_CF_SITES;
    (* Adding extra free-electron gas contribution *)
    GET_ELECTRONIC(VPT,3,A_ELEC); P_EL:=P_EL-NUM_CF_ATOMS*GAMMA_EL*A_ELEC/VPT
  END
END;

FUNCTION P_ELECTRON(VPT:EXTENDED):EXTENDED;
(* Electronic contribution to pressure. This is calculated separately because of
   many calls in volume calculations, speeding-up the process.
   See me-freepascal-15jul2016.doc, section 13 and 14
 *)
VAR A_ELEC,P_ELEC:EXTENDED;
BEGIN
  IF ELECTRONIC_YES>0 THEN
  BEGIN
    IF ELECTRONIC_YES<3 THEN
    BEGIN
      (* Free-electron-gas and extended-free-electron-gas:
         Get electronic contribution to Helmholtz energy, A_ELEC;
         section 13, eqn. (13.9)
       *)
      GET_ELECTRONIC(VPT,3,A_ELEC);
      P_ELECTRON:=-GAMMA_EL*A_ELEC/VPT
    END
    ELSE
    BEGIN
      (* Crystal-Field, section 14, eqn. (14.21) *)
      P_CRYSTAL_FIELD(VPT,P_ELEC);
      P_ELECTRON:=P_ELEC
    END
  END
  ELSE P_ELECTRON:=0
END;

END.
