UNIT isenhugo;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   This unit calculates at specific pressure the temperature on an isentrope or
   hugoniot. That is done by a minimization process of changing temperature and
   calculating the difference between entropy (energy) with the fixed entropy
   (energy). See manual chapter 17.
   The Rankine-Hugoniot energy is U=Ufix+0.5*(P+P0)*(Vfix-V) with Ufix the energy at
   ambient conditions (P0=1bar,Vfix,T_hugo_1bar). 
 *)

INTERFACE
USES CRT,sysutils,variables,makevolume;

PROCEDURE ISENHUGO_CALC(HUGO:INTEGER;U_FIX,S_FIX,V_FIX:EXTENDED;VAR V_OUT:EXTENDED);
IMPLEMENTATION

VAR COUNTMAX,VPT:EXTENDED;


PROCEDURE CHECK_COUNTER(VAR CO:INTEGER;VAR II:EXTENDED;VAR LONG:BOOLEAN);
(* Expand step after CO steps with a factor of four. That the step has been
   expanded is indicated by boolean LONG
 *)
BEGIN
  IF CO>30 THEN
  BEGIN
    IF ABS(II)<10 THEN
    BEGIN
      II:=4*II; CO:=0; LONG:=TRUE
    END
  END
END;

FUNCTION ENERGY_ENTROPY_DIF(HUGO:INTEGER;U_FIX,S_FIX,V_FIX:EXTENDED):EXTENDED;
(* Calculates entropy when isentrope (HUGO=4) or Hugoniot (HUGO=5).
   We minimize either abs(S-S_fix) or abs(U-U_hugoniot). S_fix in case of
   isentrope is the ambient entropy calculated in 'hugoniot'.
   In case of Hugoniot the calculated energy of the system should equal the
   hugoniot energy: U_fix+0.5*(P+P0)*(V_fix-V) with U_fix,V_fix at T_HUGO_1BAR and
   P0=1 bar
 *)
VAR GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,BULKMOD,DPDT,ALPHA,ZERO_PNT:EXTENDED;
BEGIN
  (* VPT is set and calculated in calc_vpt of MAKEVOLUME.PAS. The call of
     get_thermo_properties() is less efficient but simpler
   *)
  VPT:=V0; GIBBS:=0; HELMHOLTZ:=0; ENTHALPY:=0; ENERGY:=0; ENTROPY:=0;
  CP:=0; CV:=0; BULKMOD:=0; DPDT:=0; ALPHA:=0; ZERO_PNT:=0;
  GET_THERMO_PROPERTIES(VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,CP,CV,
                        BULKMOD,DPDT,ALPHA,ZERO_PNT);
  (* Isentrope *)
  IF HUGO=4 THEN ENERGY_ENTROPY_DIF:=ABS(ENTROPY-S_FIX)
  ELSE
  BEGIN
    IF HUGO=5 THEN
    BEGIN
      (* Hugoniot *)
      ENERGY_ENTROPY_DIF:=ABS(ENERGY-U_FIX-0.5*(PRESSURE+100000)*(V_FIX-VPT))
    END
  END
END;

PROCEDURE TROUBLE_TEMPERATURE;
(* Temperature has to be >0 *)
BEGIN
  IF TEMPERATURE<1E-01 THEN TEMPERATURE:=1E-01
END;

PROCEDURE SCAN_TEMPERATURE(I:EXTENDED);
BEGIN
  TEMPERATURE:=TEMPERATURE+I; TROUBLE_TEMPERATURE
END;

PROCEDURE GET_ENERGY_ENTROPY(HUGO:INTEGER;STP,U_FIX,S_FIX,V_FIX:EXTENDED);
(* In case of an isentrope (HUGO=0) calculate entropy. In case of Hugoniot (HUGO=1)
   calculate the energy and substract the Rankine-Hugoniot energy:
   U(V)-[Ufix.5*(P+P0)(Vfix-V)] with P0=1 bar, Vfix=V(T_HUGO_1BAR,1bar) and
   Ufix=U(T_HUGO_1BAR,1bar)
 *)
VAR TM,TLAST,IIN,I,SL,SN:EXTENDED;
    COUNTER:INTEGER;
    SW,LONG:BOOLEAN;
BEGIN
  TLAST:=TEMPERATURE;
  IIN:=STP; I:=IIN; LONG:=FALSE; SW:=FALSE;
  REPEAT
    TROUBLE_TEMPERATURE; TM:=TEMPERATURE; COUNTER:=0;
    IF LONG=TRUE THEN I:=I/3;
    IF (LONG=TRUE) AND (ABS(I)<=ABS(STP)) THEN LONG:=FALSE;
    IF SW=FALSE THEN SL:=ENERGY_ENTROPY_DIF(HUGO,U_FIX,S_FIX,V_FIX);
    TLAST:=TEMPERATURE; SCAN_TEMPERATURE(I); SW:=TRUE;
    SN:=ENERGY_ENTROPY_DIF(HUGO,U_FIX,S_FIX,V_FIX); TLAST:=TEMPERATURE;
    IF SN>=SL THEN
    BEGIN
      I:=-I; TEMPERATURE:=TM; TLAST:=TEMPERATURE; SN:=SL
    END;
    REPEAT
      SL:=SN; TLAST:=TEMPERATURE; SCAN_TEMPERATURE(I); COUNTER:=COUNTER+1;
      SN:=ENERGY_ENTROPY_DIF(HUGO,U_FIX,S_FIX,V_FIX); CHECK_COUNTER(COUNTER,I,LONG)
    UNTIL
      (SN>=SL) OR (COUNTER>COUNTMAX);
    TEMPERATURE:=TLAST
  UNTIL
    (LONG=FALSE) OR (COUNTER>COUNTMAX);
  STP:=IIN; TEMPERATURE:=TLAST
END;

PROCEDURE MINIM_ENERGY_ENTROPY(HUGO:INTEGER;DT,U_FIX,S_FIX,V_FIX:EXTENDED);
(* The temperature is refined in a minimization process. The volume
   VPT is global in this unit and so is the temperature T. These are
   calculated and ready at the end of this procedure
 *)
VAR STP,CRIT:EXTENDED;
BEGIN
  STP:=DT;
  IF HUGO=5 THEN CRIT:=1E-5 ELSE CRIT:=1E-6;
  WHILE ABS(STP)>CRIT DO
  BEGIN
    GET_ENERGY_ENTROPY(HUGO,STP,U_FIX,S_FIX,V_FIX); STP:=STP/3
  END
END;

PROCEDURE ISENHUGO_CALC(HUGO:INTEGER;U_FIX,S_FIX,V_FIX:EXTENDED;VAR V_OUT:EXTENDED);
(* HUGO=4=isentrope, 5=Hugoniot.
   Entering with properties that must be kept fixed. For and isentrope the entropy
   must be constant and the values U_FIX and V_FIX are meaningless: they are set to
   zero when calling this procedure. For an Hugoniot S_FIX is meaningless and set to
   zero when calling this procedure. For both cases (isentrope, Hugoniot) the
   temperature is changed. Pressure is not changed. Volume that is calculated at
   pressure and temperature is needed for calculating all thermo properties
   outside this unit.
   Returning calculated volume.  HUGO=0=isentrope, 1=Hugoniot.
   EXTRAPOLATE is used to make fast calculations based on previous calculated
   volumes. It is toggled-off here because volume jumps back and forward around a
   final value in the minimization
 *)
VAR DT:EXTENDED;
    EXTRAPOLATE_OLD:BOOLEAN;
BEGIN
  EXTRAPOLATE_OLD:=EXTRAPOLATE; EXTRAPOLATE:=FALSE;
  (* The initial step in temperature is DT *)
  VPT:=V0; DT:=10; COUNTMAX:=1E5;
  MINIM_ENERGY_ENTROPY(HUGO,DT,U_FIX,S_FIX,V_FIX);
  (* Assign calculated volume VPT to V_OUT. The global, TEMPERATURE, has been
     changed and deviates from the incoming temperature
   *)
  V_OUT:=VPT;
  EXTRAPOLATE:=EXTRAPOLATE_OLD
END;

END.
