UNIT landaucalc;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Calculates contributions due to ordering using a simple Landau theory based on that
   given by Putnis A (1992) Introduction to Mineral Sciences, Cambridge University Press,
   Cambridge. The cases treated are 1st order, 2nd order case, and tricritical.
   This unit is called by CALC_VPT() and GET_THERMO_PROPERTIES in unit MAKEVOLUME.PAS.
   Volume is changed due to Landau contribution.

   The thermodynamic properties that enter this unit are changed and passed back.

   The landau coefficients are given in XiPT as ldau[k,p,j]:
       j=2=Sm=0.5a, 3=dP/dT, 4=B, 5=c, 6=Tc0,
   (unit of the Clapeyron slope of the transition is Pascal/Kelvin)
   7=3rd law correction when 1 and no 3rd law correction when 0.
   ldau[1] contains 0, 1 or 2 expressing:
   0=if Landau must not be carried out,
   1=Landau formulation in which dG/dQ gives the correct order parameter scaling
     with (Tc-T)/Tc and Tc pressure dependent,
   ADD_TO_TOTAL=1: add the Landau contributions to all properties
   ADD_TO_TOTAL=0: do not add landau contributions: used for calculating only the Landau
                   contributions in unit thermocu.pas (XiPT).

   V_LANDAU calculates only the volume VL and corrects the entered VPT with it: the resulting
   VPT contains all contributions including Landau. This procedure is called in CALC_VPT()
   of unit MAKEVOLUME.PAS (me)

   ------------------------------------------------------------------------------------
   Correction for zero Kelvin entropy. If the correction option is set by
   ldau[7]=1 we must carry that out for the High-temperature and low-temperature
   phase: both above and below the phase transition: that is for the complete phase.
   Carry that out when the Landau contribution must be evaluated in the phase that
   is stable at 0 K and 0 Pa. E.g in Quartz(alpha), but it is not required
   for stishovite because landau in the SiO2(I,cacl2 form)
   ------------------------------------------------------------------------------------
 *)

INTERFACE
USES CRT,sysutils,variables;

PROCEDURE LANDAU_CALC(ADD_TO_TOTAL:INTEGER;VPT:EXTENDED;VAR G,H,U,S,CV,CP,AL,KT,DPDT,VL,
                      GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L:EXTENDED);
PROCEDURE VOLUME_LANDAU(ADD_TO_TOTAL:INTEGER;VAR VPT:EXTENDED);
IMPLEMENTATION

TYPE LANDAU_XIPT=ARRAY[1..7] OF EXTENDED;
VAR LDAU:LANDAU_XIPT;
    T:EXTENDED;

(* Conversion of Multiple-Einstein parameters LANDAU[1..4] to
   parameters used in XiPT, LDAU[1..7]
                  j:  1      2    3     4        5   6      7
   LDAU (XiPT):      yes/no  Sm   CS    B        C   Tc,0   3rd Law
   LANDAU (ME):       2Sm    CS   Tc,0  3rd-law           (2nd order and tricritical)
              :       Hm     CS   TR,0  Q0       3rd-law  (1st order transition)
   Note that the expressions below are based on Sm (as in XiPT)
   LDAU[1]=1
   LDAU[2]=LANDAU[1]/2;
   LDAU[3]=LANDAU[2];
   LDAU[4]=set value B for determining which Landau case (?)
   LDAU[5]=set value C for determining which Landau case (?)
   LDAU[6]=LANDAU[3];
   LDAU[7]=LANDAU[4] (for 2nd order and tricritical case)
          =LANDAU[5] (for 1st order case)
   The question (?) marks are filled with values using Putnis' expression for the
   Gibbs energy: G=0.5*a(Tc-Tc,0)*Q^2+B*Q^4/4+c*Q^6/6:
   2nd order  : LANDAU_YES=1;  B>0, c=0      : set B=10; c=0
   Tricritical: LANDAU_YES=2;  B=0, C>0      : set B=0 ; c=1
   1st order  : LANDAU_YES=3;  a>0, B<0, C>0 : set B=-1; c=1
   In XiPT values of B,C are not used directly, but the signs determine which
   Landau case should be considered.
 *)


PROCEDURE CONVERT_ME_2_XIPT;
(* Convert parameters in Multiple-Einstein input, stored in LANDAU[1..5] to
   parameters analogous to XiPT, LDAU[1..7]
 *)
VAR DELTA:EXTENDED;
BEGIN
  DELTA:=1E-08;
  LDAU[1]:=1; LDAU[2]:=LANDAU[1]/2; LDAU[3]:=LANDAU[2];
  LDAU[6]:=LANDAU[3]; LDAU[7]:=LANDAU[4];
  IF ABS(LANDAU_YES-1)<DELTA THEN
  BEGIN
    (* 2nd order case *)
    LDAU[4]:=1; LDAU[5]:=0
  END
  ELSE
  BEGIN
    IF ABS(LANDAU_YES-2)<DELTA THEN
    BEGIN
      (* Tricritical case *)
      LDAU[4]:=0; LDAU[5]:=1
    END
    ELSE
    BEGIN
      IF ABS(LANDAU_YES-3)<DELTA THEN
      BEGIN
        (* 1st order case: we must rearrange *)
        LDAU[4]:=-1; LDAU[5]:=LANDAU[4]; LDAU[7]:=LANDAU[5]
      END
    END
  END;
  (* Use variable 'T' as global for temperature *)
  T:=TEMPERATURE
END;

PROCEDURE SECOND_ORDER(VAR A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,
                       ALVL,VL_DIV_KL:EXTENDED;VAR ITS_LANDAU:BOOLEAN);
(* 2nd order transition, section 8 me-freepascal-manual.doc. dG/dQ=0 gives
   the order parameter Q2=(Tc-T)/Tc where Tc=Tc,0+CS_1*P.
   The landau coefficients are given in ldau[j]:
     j=2=Sm=0.5a, 3=dP/dT, 4=B, 5=c, 6=Tc0.
   Ldau[7]: 0=no 3rd law correction, 1=3rd law correction.
   Ldau[1]: 0=no Landau, 1=Landau formalism
   See me-freepascal-15jul2016.doc
 *)
VAR CS_1,DELTA,TC,Q:EXTENDED;
    LDAU7:INTEGER;
BEGIN
  (* Inverse Clapeyron slope is CS_1 *)
  CS_1:=1/LDAU[3]; DELTA:=1E-08;
  (* Round ldau[7]: 1=3rd law correction; 0=no 3rd law correction *)
  LDAU7:=ROUND(LDAU[7]);
  (* Critical temperature, eqn. (7.11) *)
  TC:=LDAU[6]+CS_1*PRESSURE;
  IF T<TC-DELTA THEN
  BEGIN
    (* Order parameter, eqn. (8.5) *)
    Q:=SQRT(1-T/TC);
    (* Gibbs energy, eqn. (8.8) *)
    GL:=-A*TC*INT_POW_N(Q,4)/4;
    (* Gibbs energy: 3rd law correction: see analogous to eqn. (8.17) *)
    IF LDAU7=1 THEN GL:=GL-LDAU[2]*T;
    (* Entropy, eqn.(8.10) *)
    SL:=-A*Q*Q/2;
    (* Entropy: 3rd law correction, eqn. (8.16) *)
    IF LDAU7=1 THEN SL:=SL+LDAU[2];
    (* Enthalpy, eqn. (8.11) *)
    HL:=GL+T*SL;
    (* Heat capacity, eqn. (8.12) *)
    CP_L:=A*(1-Q*Q)/2;
    (* Volume, eqn. (8.13) *)
    VL:=-A*Q*Q*(1-Q*Q/2)*CS_1/2;
    (* Energy *)
    UL:=HL-PRESSURE*VL;
    (* Quotient of volume VL and bulkmodulus KL, eqn. (8.14) *)
    VL_DIV_KL:=A*T*CS_1*CS_1/TC/TC/2; VL_DIV_KL:=VL_DIV_KL*(1-Q*Q);
    (* Isothermal Bulk modulus, contribution not used *)
    IF T>DELTA THEN KT_L:=VL/VL_DIV_KL ELSE KT_L:=0;
    (* Product of expansivity and volume, eqn. (8.15) *)
    ALVL:=A*CS_1/TC/2; ALVL:=ALVL*(1-Q*Q);
    (* Expansivity, contribution not used *)
    AL_L:=ALVL/VL;
    (* Isochoric heat capacity, contribution not used *)
    CV_L:=CP_L-AL_L*AL_L*KT_L*VL*T;
    (* dP/dT)V, contribution not used *)
    DPDT_L:=AL_L*KT_L
  END
  ELSE
  BEGIN
    (* Temperature above critical temperature, therefore no Landau contribution *)
    ITS_LANDAU:=FALSE
  END
END;

PROCEDURE TRICRITICAL(VAR A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,
                      ALVL,VL_DIV_KL:EXTENDED;VAR ITS_LANDAU:BOOLEAN);
(* Tricritical transition, section 9 me-freepascal-manual.doc. dG/dQ=0 gives
   the order parameter Q2=(Tc-T)/Tc where Tc=Tc,0+CS_1*P.
   The landau coefficients are given in ldau[j]:
     j=2=Sm=0.5a, 3=dP/dT, 4=B, 5=c, 6=Tc0.
   Ldau[7]: 0=no 3rd law correction, 1=3rd law correction.
   Ldau[1]: 0=no Landau, 1=Landau
   See me-freepascal-15jul2016.doc
 *)
VAR CS_1,DELTA,TC,Q,EX:EXTENDED;
    LDAU7:INTEGER;
BEGIN
  (* Inverse Clapeyron slope is CS_1 *)
  CS_1:=1/LDAU[3]; DELTA:=1E-08;
  (* Round ldau[7]: 1=3rd law correction; 0=no 3rd law correction *)
  LDAU7:=ROUND(LDAU[7]);
  (* Critical temperature, eqn. (7.11) *)
  TC:=LDAU[6]+CS_1*PRESSURE;
  IF T<TC-DELTA THEN
  BEGIN
    (* Order parameter, eqn. (9.5) *)
    EX:=1/4; Q:=1-T/TC; Q:=POW_N(Q,EX);
    (* Gibbs energy, eqn. (9.7) *)
    GL:=-A*TC*INT_POW_N(Q,6)/3;
    (* Gibbs energy: 3rd law correction: see analogous to eqn.(9.17) *)
    IF LDAU7=1 THEN GL:=GL-LDAU[2]*T;
    (* Entropy, eqn. (9.10) *)
    SL:=-A*Q*Q/2;
    (* Entropy: 3rd law correction, eqn. (9.16) *)
    IF LDAU7=1 THEN SL:=SL+LDAU[2];
    (* Enthalpy, eqn. (9.11) *)
    HL:=GL+T*SL;
    (* Heat capacity, eqn. (9.12) *)
    CP_L:=A*T/4/TC/Q/Q;
    (* Volume, eqn. (9.13) *)
    VL:=-A*Q*Q*CS_1/2;
    VL:=VL*(1-INT_POW_N(Q,4)/3);
    (* Energy, contribution not used *)
    UL:=HL-PRESSURE*VL;
    (* Quotient of volume VL and bulkmodulus KL: VL/KL=-dVl/dP, eqn. (9.14) *)
    VL_DIV_KL:=A*T*CS_1*CS_1/TC/TC/4; VL_DIV_KL:=VL_DIV_KL*(1/Q/Q-Q*Q);
    (* Isothermal Bulk modulus, contribution not used *)
    IF T>DELTA THEN KT_L:=VL/VL_DIV_KL ELSE KT_L:=0;
    (* Product of expansivity and volume, eqn. (9.15) *)
    ALVL:=A*CS_1/TC/4; ALVL:=ALVL*(1/Q/Q-Q*Q);
    (* Expansivity, contribution not used *)
    AL_L:=ALVL/VL;
    (* Isochoric heat capacity, contribution not used *)
    CV_L:=CP_L-AL_L*AL_L*KT_L*VL*T;
    (* dP/dT)V, contribution not used *)
    DPDT_L:=AL_L*KT_L
  END
  ELSE
  BEGIN
    (* Temperature above critical temperature, therefore no Landau contribution *)
    ITS_LANDAU:=FALSE
  END
END;

PROCEDURE FIRST_ORDER(VAR A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,
                      ALVL,VL_DIV_KL:EXTENDED;VAR ITS_LANDAU:BOOLEAN);
(* 1st order transition
   The landau coefficients are given in ldau[j]: j=2=Sm=, 3=CS=dP/dT, 4=B, 5=c, 6=Tc0.
   Ldau[7]: 0=no 3rd law correction, 1=3rd law correction.
   Ldau[1]: 0=no Landau, 1=Landau on
 *)
VAR CS_1,DELTA,HM,TR,Q,Q0,Q2,HULP,HULP2,DQDP,DCDP:EXTENDED;
    LDAU7:INTEGER;
BEGIN
  CS_1:=1/LDAU[3]; Q0:=LDAU[5]; DELTA:=1E-12;
  (* Round the extended real ldau[7]: 1=3rd law correction, 0=no 3rd law correction *)
  LDAU7:=ROUND(LDAU[7]);
  (* Transition temperature, eqn. (10.12) *)
  TR:=LDAU[6]+CS_1*PRESSURE;
  IF T<TR-DELTA THEN
  BEGIN
    (* First order transition: constants are 'a', 'Q0'. Parameter 'c' is pressure
       dependent. When Q0=0 we have the tricritical case
     *)
    IF Q0<0 THEN Q0:=0;
    (* HM is the transition enthalpy at P=0, CS_1 is the inverse Clapeyron slope *)
    HM:=0.5*A*TR*Q0*Q0;
    (* Parameter 'a' must always be positive *)
    IF A<DELTA THEN A:=DELTA;
    (* Determine 'c' and (dc/dP) from eqn. (10.17) *)
    HULP:=1-2*Q0*Q0/3; HULP:=HULP*HULP-INT_POW_N(Q0,4)/9;
    C:=A*TR/HULP;
    DCDP:=A*CS_1/HULP;
    (* Determine Q^2 from (10.12): NOTE by taking powers that Q2=Q*Q *)
    Q2:=2*Q0*Q0/3;
    HULP:=INT_POW_N(Q0,4)/9+A*(TR-T)/C;
    HULP:=SQRT(HULP);
    Q2:=Q2+HULP;
    IF (Q2>=Q0*Q0) AND (Q2<=1) THEN ITS_LANDAU:=TRUE ELSE ITS_LANDAU:=FALSE;
    IF ITS_LANDAU=TRUE THEN
    BEGIN
      (* Take sqrt(Q^2) to have the value of Q because we calculated Q-squared *)
      Q:=SQRT(Q2);
      (* Entropy: eqn. (10.15) *)
      SL:=-0.5*A*Q2;
      (* 3rd law correction for entropy when LDAU7=1 *)
      IF LDAU7=1 THEN SL:=SL+0.5*A;
      (* Gibbs energy: eqn. (10.10): note that Q2=Q*Q when taking powers below *)
      HULP:=T-TR+INT_POW_N(Q0,4)*C/3/A;
      GL:=0.5*A*HULP*Q2;
      GL:=GL-Q0*Q0*C*Q2*Q2/3;
      GL:=GL+C*INT_POW_N(Q2,3)/6;
      (* 3rd law correction when LDAU7=1 *)
      IF LDAU7=1 THEN GL:=GL-0.5*A*T;
      (* Enthalpy *)
      HL:=GL+T*SL;
      (* Heat capacity, eqn. (10.21) *)
      HULP:=INT_POW_N(Q0,4)/9+A*(TR-T)/C; HULP:=SQRT(HULP); HULP:=1/HULP;
      CP_L:=A*A*T*HULP/4/C;
      (* Volume, EQN. (10.19) *)
      HULP:=-CS_1+INT_POW_N(Q0,4)*DCDP/3/A; VL:=A*HULP*Q2/2;
      VL:=VL-Q0*Q0*DCDP*INT_POW_N(Q2,2)/3;
      VL:=VL+DCDP*INT_POW_N(Q2,3)/6;
      (* Energy *)
      UL:=HL-PRESSURE*VL;
      (* Product of expansivity and volume: dVL/dT, eqn.(10.26) *)
      HULP:=INT_POW_N(Q0,4)/9+A*(TR-T)/C; HULP:=SQRT(HULP); HULP:=1/HULP;
      ALVL:=A*A*HULP*(CS_1*C-(TR-T)*DCDP)/4/C/C;
      (* Expansivity *)
      AL_L:=ALVL/VL;
      (* Quotient of volume VL and bulkmodulus KL: -dV/dP, eqn.(10.28) *)
      HULP:=INT_POW_N(Q0,4)/9+A*(TR-T)/C; HULP:=SQRT(HULP); HULP:=1/HULP;
      DQDP:=A*HULP*(CS_1*C-(TR-T)*DCDP)/4/Q/C/C;
      HULP:=-CS_1+INT_POW_N(Q0,4)*DCDP/3/A;
      HULP2:=A*HULP-4*Q0*Q0*DCDP*Q2/3+DCDP*Q2*Q2;
      VL_DIV_KL:=-HULP2*Q*DQDP;
      (* Isothermal Bulk modulus *)
      IF T>DELTA THEN KT_L:=VL/VL_DIV_KL ELSE KT_L:=0;
      (* Isochoric heat capacity *)
      CV_L:=CP_L-AL_L*AL_L*KT_L*VL*T;
      (* dP/dT)V *)
      DPDT_L:=AL_L*KT_L
    END
  END
  ELSE
  BEGIN
    (* Temperature above critical temperature, therefore no Landau contribution *)
    ITS_LANDAU:=FALSE
  END
END;

PROCEDURE LANDAU_CALC(ADD_TO_TOTAL:INTEGER;VPT:EXTENDED;VAR G,H,U,S,CV,CP,AL,KT,DPDT,
                      VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L:EXTENDED);
(* The landau coefficients are given in ldau[k,p,j]:
   2nd order and tricritical: j=2=Sm=0.5a, 3=dP/dT, 4=B, 5=c, 6=Tc,0.
   1st order                : j=2=Sm=0.5a, 3=dP/dT, 4=B, 5=Q0,6=Tr,0.
   And LDAU[1] discriminates as:
   0=No Landau contribution
   1=Using Landau method in which Q derived from dG/dQ scales with (Tc-T)/Tc
     (for 1st order it scales in a more complicated manner, see me-freepascal.doc)
   There are 3 cases considered:
   a    B    c(or Q0)
        >0   0   2nd order
         0   >0  tricritical
   >0   <0   >0  1st order
   When values do not meet one of these 3 conditions we don't add Landau contributions                                   
   See me-freepascal-14sep2017.doc
 *)
VAR A,B,C,DELTA,HULP,ALVL,VL_DIV_KL,VPT_NO_LANDAU:EXTENDED;
    ITS_LANDAU:BOOLEAN;
    LDAU1,LDAU7:INTEGER;
BEGIN
  (* Convert Multiple-Einstein input to parameters analogous in XiPT *)
  CONVERT_ME_2_XIPT;
  (* Initialize Landau properties *)
  VL:=0; GL:=0; HL:=0; UL:=0; SL:=0; CV_L:=0; CP_L:=0; AL_L:=0; KT_L:=0; DPDT_L:=0;
  DELTA:=1E-12; ITS_LANDAU:=FALSE;
  LDAU1:=ROUND(LDAU[1]); LDAU7:=ROUND(LDAU[7]);
  (* Use same coefficients:
     2nd order and tricritical: a=2*Sm=2*ldau[2], B=ldau[4], c=ldau[5]
     1st order                : a=Sm=ldau[2],     B=ldau[4], Q0=ldau[5]
   *)
  A:=2*LDAU[2]; B:=LDAU[4]; C:=LDAU[5];
  IF (B>DELTA) AND (ABS(C)<DELTA) THEN
  BEGIN
    (* 2nd order transition, section 8 *)
    ITS_LANDAU:=TRUE;
    IF LDAU1=1 THEN
    BEGIN
      SECOND_ORDER(A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,ALVL,VL_DIV_KL,ITS_LANDAU)
    END
  END
  ELSE
  BEGIN
    IF (ABS(B)<DELTA) AND (C>DELTA) THEN
    BEGIN
      (* Tricritical, section 9 *)
      ITS_LANDAU:=TRUE;
      IF LDAU1=1 THEN
      BEGIN
        TRICRITICAL(A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,ALVL,VL_DIV_KL,ITS_LANDAU)
      END
    END
    ELSE
    BEGIN
      IF (A>DELTA) AND (B<-DELTA) AND (C>DELTA) THEN
      BEGIN
        (* 1st order transition, section 10 *)
        ITS_LANDAU:=TRUE;
        FIRST_ORDER(A,B,C,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,ALVL,VL_DIV_KL,ITS_LANDAU)
      END
      ELSE
      BEGIN
        (* No conditions met with, so do nothing *)
        ITS_LANDAU:=FALSE
      END
    END
  END;
  (* ITS_LANDAU=true when one of the 3 cases are found and when temperature is below the
     pressure dependent critical temperature
   *)
  IF ITS_LANDAU=TRUE THEN
  BEGIN
    (* Add Landau contributions to thermodynamic properties when add_to_total=1.
       If add_to_total=0 we are only interested in the Landau properties themselves
     *)
    IF ADD_TO_TOTAL=1 THEN
    BEGIN
      (* Gibbs energy *)
      G:=G+GL;
      (* Enthalpy *)
      H:=H+HL;
      (* Entropy *)
      S:=S+SL;
      (* Heat capacity *)
      CP:=CP+CP_L;
      (* Volume must not be corrected because that has been done already in
         CALC_VPT() in unit MAKEVOLUME.PAS.
         For K and alpha however we need the contributions without Landau,
         vpt_no_landau. The reason is that the total volume due to all
         contributions including landau contribution is included in VPT which enters
         this unit. VPT also includes the 3rd law correction for 1st order when
         that has ben indicated in the input file.
       *)
      VPT_NO_LANDAU:=VPT-VL;
      (* Energy *)
      U:=H-PRESSURE*VPT;
      (* Expansivity, VPT includes the Landau contribution already *)
      AL:=(AL*VPT_NO_LANDAU+ALVL)/VPT;
      (* Isothermal bulkmodulus, VPT includes the Landau contribution already *)
      HULP:=VPT_NO_LANDAU/KT+VL_DIV_KL;
      KT:=VPT/HULP;
      (* dP/dT)V = alpha*K *)
      DPDT:=AL*KT;
      (* Isochoric heat capacity *)
      CV:=CP-AL*AL*KT*VPT*T
    END
  END
  ELSE
  BEGIN
    (* ---------------------HERE ITS_LANDAU=FALSE---------------------------------
       Correction for the zero Kelvin entropy FOR ONLY THE HT PHASE. The LT phase
       has been done already. If the correction option is set by ldau[7]=1 we must
       carry that out for the High-temperature. Carry that out when the Landau
       contribution must be evaluated in the phase that is stable at 0 K and 0 Pa.
       Eg in alpha-quartz but not in stishovite
       ---------------------------------------------------------------------------
     *)
    IF LDAU7=1 THEN
    BEGIN
      (* 2nd order, tricritical and 1st order case: correction is the same.
         Add +0.5*a=Sm to the entropy. Only entropy, energy, enthalpy,
         Gibbs energy are affected: eqn. (8.16, 8.17, 9.16, 9.17, 10.34, 10.36)
       *)
      SL:=LDAU[2]; GL:=-LDAU[2]*T; HL:=GL+T*SL; UL:=HL-PRESSURE*VPT;
      IF ADD_TO_TOTAL=1 THEN
      BEGIN
        S:=S+SL; G:=G+GL; H:=G+T*S; U:=H-PRESSURE*VPT
      END
    END
  END
END;

PROCEDURE V_LANDAU(TYP:INTEGER;A,B,C:EXTENDED;VAR VL:EXTENDED;VAR ITS_LANDAU:BOOLEAN);
(* Calculation of volume contribution due to Landau formalism.
   TYP=1=second order, 2=tricritical, 3=first order
   In the method: dG/dQ=0 gives the order parameter Q described in terms
   of (Tc-T)/Tc for the 2nd order and tricritical case: Tc=Tc,0+(Vm/Sm)P=Tc,0+CS_1*P.
   See me-freepascal-15jul2016.doc
 *)
VAR CS_1,DELTA,HM,Q,TC,TR,Q0,Q2,HULP,EX,DCDP:EXTENDED;
BEGIN
  (* Inverse Clapeyron slope is CS_1 *)
  CS_1:=1/LDAU[3]; DELTA:=1E-12; VL:=0;
  (* Critical/Transition temperature *)
  TC:=LDAU[6]+CS_1*PRESSURE;
  IF T<TC-DELTA THEN
  BEGIN
    IF TYP=1 THEN
    BEGIN
      (* Order parameter for 2nd order, eqn. (8.5) *)
      Q:=SQRT(1-T/TC);
      (* Volume, eqn. (8.13) *)
      VL:=-A*Q*Q*CS_1/2; VL:=VL*(1-Q*Q/2); ITS_LANDAU:=TRUE
    END;
    IF TYP=2 THEN
    BEGIN
      (* Order parameter for tricritical, eqn. (9.5) *)
      EX:=1/4;
      Q:=1-T/TC; Q:=POW_N(Q,EX);
      (* Volume, eqn. (9.13) *)
      VL:=-A*Q*Q*CS_1/2;
      VL:=VL*(1-INT_POW_N(Q,4)/3);
      ITS_LANDAU:=TRUE
    END;
    IF TYP=3 THEN
    BEGIN
      (* First order transition, CS_1 is the inverse Clapeyron slope
       *)
      TR:=TC; Q0:=LDAU[5];
      IF Q0<0 THEN Q0:=0;
      (* First order transition: constants are 'a', 'Q0'. Parameter 'c' is pressure
         dependent. When Q0=0 we have the tricritical case
       *)
      HM:=0.5*A*TR*Q0*Q0;
      (* Parameter 'a' must be positive always *)
      IF A<DELTA THEN A:=DELTA;
      (* Determine 'c' and (dc/dP) from eqn. (10.17) *)
      HULP:=1-2*Q0*Q0/3; HULP:=HULP*HULP-INT_POW_N(Q0,4)/9;
      C:=A*TR/HULP;
      DCDP:=A*CS_1/HULP;
      (* Determine Q^2 from (10.12): NOTE by taking powers that Q2=Q*Q *)
      Q2:=2*Q0*Q0/3;
      HULP:=INT_POW_N(Q0,4)/9+A*(TR-T)/C;
      HULP:=SQRT(HULP);
      Q2:=Q2+HULP;
      (* Volume *)
      HULP:=-CS_1+INT_POW_N(Q0,4)*DCDP/3/A; VL:=A*HULP*Q2/2;
      VL:=VL-Q0*Q0*DCDP*INT_POW_N(Q2,2)/3;
      VL:=VL+DCDP*INT_POW_N(Q2,3)/6;
      ITS_LANDAU:=TRUE
    END
  END
  ELSE
  BEGIN
    (* Temperature above critical temperature, therefore no Landau contribution *)
    ITS_LANDAU:=FALSE
  END
END;

PROCEDURE VOLUME_LANDAU(ADD_TO_TOTAL:INTEGER;VAR VPT:EXTENDED);
(* V_LANDAU calculates only the volume VL and corrects the entered VPT with it:
   the resulting volume, VPT, contains all contributions including Landau.
   This procedure is called in CALC_VPT() of unit MAKEVOLUME.PAS.
   In the tricritical case:
   LDAU1=1: Gl=0.5a(T-Tc)Q^2+(1/6)cQ^6
*)
VAR A,B,C,DELTA,VL:EXTENDED;
    ITS_LANDAU:BOOLEAN;
    LDAU1:INTEGER;
BEGIN
  (* Convert Multiple-Einstein input to variables used in XiPT *)
  CONVERT_ME_2_XIPT;
  LDAU1:=ROUND(LDAU[1]);
  (* Check if Landau theory must be applied *)
  IF LDAU1>=1 THEN
  BEGIN
    IF ADD_TO_TOTAL=1 THEN
    BEGIN
      DELTA:=1E-12; ITS_LANDAU:=FALSE;
      (* Use same coefficients as in me-freepascal.doc: a=2*Sm=2*ldau[2], B=ldau[4],
         c=ldau[5]
       *)
      A:=2*LDAU[2]; B:=LDAU[4]; C:=LDAU[5];
      IF (B>DELTA) AND (ABS(C)<DELTA) THEN
      BEGIN
        (* 2nd order transition. *)
        ITS_LANDAU:=TRUE;
        V_LANDAU(1,A,B,C,VL,ITS_LANDAU)
      END
      ELSE
      BEGIN
        IF (ABS(B)<DELTA) AND (C>DELTA) THEN
        BEGIN
          (* Tricritical *)
          ITS_LANDAU:=TRUE;
          V_LANDAU(2,A,B,C,VL,ITS_LANDAU)
        END
        ELSE
        BEGIN
          IF (A>DELTA) AND (B<-DELTA) AND (C>DELTA) THEN
          BEGIN
            (* 1st order transition *)
            ITS_LANDAU:=TRUE;
            V_LANDAU(3,A,B,C,VL,ITS_LANDAU)
          END
          ELSE
          BEGIN
            (* No conditions met with, so do nothing *)
            ITS_LANDAU:=FALSE
          END
        END
      END;
      (* If one of the three cases applies and when temperature below the critical
         temperature only then correct the volume VPT due to Landau contribution
       *)
      IF ITS_LANDAU=TRUE THEN VPT:=VPT+VL;
    END
  END
END;

END.
