UNIT makevolume;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Calculates thermodynamic properties in the multiple Einstein method:
   - Equation of State is in PRESSURE_EQUATION()
   - Total shear modulus (static+vibrational) in SHEAR_MODULUS(),
   - Volume is calculated in CALC_VPT(),
   - Total thermodynamic properties (static+vib+electronic+magnetic):
       GET_THERMO_PROPERTIES()
   - Correcting volume of static lattice when model parameters are changed
       FIX_STATIC_VOLUME()
 *)

INTERFACE
USES CRT,sysutils,variables,readinput,static,vibrations,landaucalc,electronic,
     magnetic,cationdisorder,defects;


FUNCTION SHEAR_MODULUS(VPT:EXTENDED):EXTENDED;
PROCEDURE CALC_VPT(VAR VPT:EXTENDED);
PROCEDURE GET_THERMO_PROPERTIES(VAR VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,
                                CP,CV,BULKMOD,DPDT,ALPHA,ZERO_PNT:EXTENDED);
PROCEDURE FIX_STATIC_VOLUME(VAR V0_ST_FIX:EXTENDED;VAR FIX_YES:BOOLEAN);
IMPLEMENTATION

VAR P_ZEROVIB:EXTENDED;  (* P_ZEROVIB: thermal pressure value at V0, T=0 *)


FUNCTION SHEAR_MODULUS(VPT:EXTENDED):EXTENDED;
(* shear modulus: the sum of static and vibrational contributions *)
VAR Z,GSH_VIB,V_TOTAL,V_LANDAU:EXTENDED;
BEGIN
  (* Shear modulus is programmed when the static equation of state is finite
     strain: EOS_TYPE=2
   *)
  IF EOS_TYPE=2 THEN
  BEGIN
    V_TOTAL:=VPT;
    (* In case of Landau contributions: we must use volume without Landau.
       We enter this function with the total volume including Landau contribution.
       See me-freepascal.doc, section 7
     *)
    IF LANDAU_YES>1E-8 THEN
    BEGIN
      (* VOLUME_LANDAU() is in unit LANDAUCALC.PAS *)
      V_LANDAU:=0; VOLUME_LANDAU(1,V_LANDAU);
      (* Now make volume without Landau contribution *)
      VPT:=VPT-V_LANDAU
    END;
    (* Static contribution, unit STATIC.PAS *)
    Z:=STATIC_SHEARMODULUS(VPT);
    (* Add vibrational contribution present in unit VIBRATIONS.PAS *)
    GSH_VIB:=0; SHEAR_VIB(VPT,GSH_VIB);
    Z:=Z+GSH_VIB;
    IF Z>0 THEN SHEAR_MODULUS:=Z ELSE SHEAR_MODULUS:=0;
    (* Reset volume to that including Landau contribution *)
    VPT:=V_TOTAL
  END
  ELSE
  BEGIN
    (* Shear modulus has not been programmed for EoS that is not based
       on finite strain, therefore it is set to zero
     *)
    SHEAR_MODULUS:=0
  END
END;

FUNCTION PRESSURE_EQUATION(VPT:EXTENDED):EXTENDED;
(* Equation of state containing all physical contributions *)
TYPE RY_BIRCH=ARRAY[2..MAX_EOS_ORDER] OF EXTENDED;
VAR BMN:RY_BIRCH;
    ETA,EXPON,P_STATIC,P_TOTAL,ZY,Z1,Z2,STRAIN,SUM_F:EXTENDED;
    J:INTEGER;
BEGIN
  (* Functions POW_N, INT_POW_N are in unit VARIABLES.PAS *)
  CASE EOS_TYPE OF
    1:BEGIN
        (* 3rd order Vinet equation of state, see unit STATIC.PAS for Helmholtz energy
           and bulk modulus. See FreePascal Manual Eq. (5.14)
         *)
        EXPON:=1/3; ETA:=V0_ST/VPT;
        Z1:=3*K0_ST*POW_N(ETA,EXPON)*(POW_N(ETA,EXPON)-1);
        EXPON:=-1/3;
        Z2:=1.5*(DK0_ST[1]-1)*(1-POW_N(ETA,EXPON));
        IF Z2>MAX_ARG_EXP THEN Z2:=MAX_ARG_EXP
        ELSE
          IF Z2<-MAX_ARG_EXP THEN Z2:=-MAX_ARG_EXP;
        P_STATIC:=Z1*EXP(Z2);
        IF GET_V0ST=FALSE THEN
        BEGIN
          (* Normal calculation: no correction for static volume.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             Other contributions in units VIBRATIONS.PAS, ELECTRONIC.PAS, DEFECTS.PAS
           *)
          P_TOTAL:=P_STATIC+THERMAL_PRESSURE(VPT)+P_ELECTRON(VPT)+DEFECT_PRESSURE(VPT)
        END
        ELSE
        BEGIN
          (* Special case: correction for static volume required.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             P_ZEROVIB is thermal_pressure at V0, T=0.
             At T=0 and V=V0 the electronic pressure is zero for free-electron gas
             (and extended) and crystal-field models. Pressure due to mono vacancy defects
             is also zero.
           *)
          P_TOTAL:=P_STATIC+P_ZEROVIB
        END;
        IF ATTRAC_REP=FALSE THEN PRESSURE_EQUATION:=ABS(PRESSURE-P_TOTAL)
        ELSE PRESSURE_EQUATION:=P_TOTAL
      END;
    2:BEGIN
        ETA:=V0_ST/VPT; EXPON:=1/3;
        (* Birch-Murnaghan, 3rd order and higher order *)
        IF EOS_ORDER=3 THEN
        BEGIN
          (* 3rd order Birch-Murnaghan EOS, see unit STATIC.PAS for Helmholtz energy
             and bulk modulus.
           *)
          ZY:=POW_N(ETA,EXPON);
          Z1:=POW_N(ZY,7)-POW_N(ZY,5);
          Z2:=1-0.75*(4-DK0_ST[1])*(ZY*ZY-1);
          P_STATIC:=1.5*K0_ST*Z1*Z2;
          IF GET_V0ST=FALSE THEN
          BEGIN
            (* Normal calculation: no correction for static volume.
               GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit
               Other contributions in units VIBRATIONS.PAS, ELECTRONIC.PAS, DEFECTS.PAS
             *)
            P_TOTAL:=P_STATIC+THERMAL_PRESSURE(VPT)+P_ELECTRON(VPT)+DEFECT_PRESSURE(VPT)
          END
          ELSE
          BEGIN
            (* Special case: correction for static volume required.
               GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
               P_ZEROVIB is thermal_pressure at V0, T=0.
               At T=0 and V=V0 the electronic pressure is zero for free-electron gas
               (and extended) and crystal-field models. Pressure due to mono vacancy defects
               is also zero.
             *)
            P_TOTAL:=P_STATIC+P_ZEROVIB
          END;
          IF ATTRAC_REP=FALSE THEN PRESSURE_EQUATION:=ABS(PRESSURE-P_TOTAL)
          ELSE PRESSURE_EQUATION:=P_TOTAL
        END
        ELSE
        BEGIN
          (* N-th order BM EoS with strain coefficients 2 to EOS_ORDER.
             BMN[2]=1. There are 1..EOS_ORDER-2 coefficients dealing with
             derivatives of bulk modulus, DK0_ST[J]. Strain is 'STRAIN'.
             Unit STATIC.PAS. See FreePascal Manual Eq. (5.8)
           *)
          ETA:=VPT/V0_ST; EXPON:=-2/3; STRAIN:=(POW_N(ETA,EXPON)-1)/2;
          FOR J:=1 TO EOS_ORDER-2 DO BMN[J+2]:=DK0_ST[J];
          BMN[2]:=1; SUM_F:=0;
          FOR J:=2 TO EOS_ORDER DO SUM_F:=SUM_F+BMN[J]*INT_POW_N(STRAIN,J-1);
          EXPON:=5/2; Z1:=1+2*STRAIN; Z1:=POW_N(Z1,EXPON);
          P_STATIC:=3*K0_ST*Z1*SUM_F;
          IF GET_V0ST=FALSE THEN
          BEGIN
            (* Normal calculation: no correction for static volume.
               GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit
               Other contributions in units VIBRATIONS.PAS, ELECTRONIC.PAS, DEFECTS.PAS
             *)
            P_TOTAL:=P_STATIC+THERMAL_PRESSURE(VPT)+P_ELECTRON(VPT)+DEFECT_PRESSURE(VPT)
          END
          ELSE
          BEGIN
            (* Special case: correction for static volume required.
               GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
               P_ZEROVIB is thermal_pressure at V0, T=0.
               At T=0 and V=V0 the electronic pressure is zero for free-electron gas
               (and extended) and crystal-field models. Pressure due to mono vacancy defects
               is also zero.
             *)
            P_TOTAL:=P_STATIC+P_ZEROVIB
          END;
          IF ATTRAC_REP=FALSE THEN PRESSURE_EQUATION:=ABS(PRESSURE-P_TOTAL)
          ELSE PRESSURE_EQUATION:=P_TOTAL
        END
      END;
   3:BEGIN
        (* Keane's equation of state, see unit STATIC.PAS for Helmholtz energy
           and bulk modulus. See FreePascal Manual Eq. (5.19)
         *)
        EXPON:=-DK0_ST[2]; ETA:=VPT/V0_ST; Z1:=POW_N(ETA,EXPON);
        Z2:=DK0_ST[1]*(Z1-1)/DK0_ST[2]/DK0_ST[2];
        ZY:=DK0_ST[1]/DK0_ST[2]-1;
        ZY:=ZY*LN(ETA);
        P_STATIC:=K0_ST*(Z2+ZY);
        IF GET_V0ST=FALSE THEN
        BEGIN
          (* Normal calculation: no correction for static volume.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             Other contributions in units VIBRATIONS.PAS, ELECTRONIC.PAS, DEFECTS.PAS
           *)
          P_TOTAL:=P_STATIC+THERMAL_PRESSURE(VPT)+P_ELECTRON(VPT)+DEFECT_PRESSURE(VPT)
        END
        ELSE
        BEGIN
          (* Special case: correction for static volume required.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             P_ZEROVIB is thermal_pressure at V0, T=0.
             At T=0 and V=V0 the electronic pressure is zero for free-electron gas
             (and extended) and crystal-field models. Pressure due to mono vacancy defects
             is also zero.
           *)
          P_TOTAL:=P_STATIC+P_ZEROVIB
        END;
        IF ATTRAC_REP=FALSE THEN PRESSURE_EQUATION:=ABS(PRESSURE-P_TOTAL)
        ELSE PRESSURE_EQUATION:=P_TOTAL
      END;
    4:BEGIN
        (* EoS of Qin et al (2008), see unit STATIC.PAS for Helmholtz energy
           and bulk modulus. See FreePascal Manual Eq. (5.24)
         *)
        ETA:=VPT/V0_ST; EXPON:=1/3; STRAIN:=POW_N(ETA,EXPON);
        SUM_F:=(DK0_ST[2]-DK0_ST[4])*POW_N(STRAIN,-DK0_ST[3]);
        SUM_F:=SUM_F*(DK0_ST[1]+DK0_ST[3]/STRAIN)*EXP(-DK0_ST[1]*(STRAIN-1));
        Z1:=-(DK0_ST[1]+DK0_ST[3])*POW_N(STRAIN,DK0_ST[4]);
        Z1:=Z1*(DK0_ST[2]-DK0_ST[4]/STRAIN)*EXP(-DK0_ST[2]*(STRAIN-1));
        SUM_F:=SUM_F+Z1;
        Z1:=(DK0_ST[1]+DK0_ST[3])*(DK0_ST[2]-DK0_ST[4]);
        Z1:=Z1*(DK0_ST[1]+DK0_ST[3]-DK0_ST[2]+DK0_ST[4]);
        Z1:=Z1+DK0_ST[1]*DK0_ST[4]+DK0_ST[2]*DK0_ST[3];
        Z1:=Z1*STRAIN*STRAIN;
        Z1:=3*K0_ST/Z1;
        P_STATIC:=Z1*SUM_F;
        IF GET_V0ST=FALSE THEN
        BEGIN
          (* Normal calculation: no correction for static volume.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             Other contributions in units VIBRATIONS.PAS, ELECTRONIC.PAS, DEFECTS.PAS
           *)
          P_TOTAL:=P_STATIC+THERMAL_PRESSURE(VPT)+P_ELECTRON(VPT)+DEFECT_PRESSURE(VPT)
        END
        ELSE
        BEGIN
          (* Special case: correction for static volume required.
             GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit.
             P_ZEROVIB is thermal_pressure at V0, T=0.
             At T=0 and V=V0 the electronic pressure is zero for free-electron gas
             (and extended) and crystal-field models. Pressure due to mono vacancy defects
             is also zero.
           *)
          P_TOTAL:=P_STATIC+P_ZEROVIB
        END;
        IF ATTRAC_REP=FALSE THEN PRESSURE_EQUATION:=ABS(PRESSURE-P_TOTAL)
        ELSE PRESSURE_EQUATION:=P_TOTAL
      END
  END {of case}
END;

PROCEDURE TROUBLE_VOLUME(VAR VPT,STP:EXTENDED);
(* Set a lowest value for the volume. If the step is larger than the
   volume/10 we set the step to vpt/10
 *)
VAR SIGN:INTEGER;
BEGIN
  IF VPT<1E-08 THEN VPT:=1E-08;
  IF STP<0 THEN SIGN:=-1 ELSE SIGN:=1;
  IF ABS(STP)>VPT/10 THEN STP:=SIGN*VPT/10
END;

PROCEDURE EXPAND_STEP(VAR CO:INTEGER;VAR II:EXTENDED;VAR LONG_V:BOOLEAN);
(* If after CO steps in the repeat-until scan in volume not a minimum has been
   achieved, increase the step with a factor 3, to speed-up the search.
 *)
BEGIN
  IF CO>8 THEN
  BEGIN
    IF ABS(II)<1E-3 THEN
    BEGIN
      II:=3*II; CO:=0; LONG_V:=TRUE
    END
  END;
  (* Reset counter *)
  IF CO>30000 THEN CO:=0
END;

PROCEDURE ESTIMATE_VPT_STEP(VAR VPT,STP:EXTENDED);
(* Estimate the next volume based on preceding end results of volume stored in VMEMORY
   and OLD_VMEMORY. From these also follows an estimated step in volume
 *)
VAR D,SIGN:EXTENDED;
BEGIN
  IF EXTRAPOLATE=TRUE THEN
  BEGIN
    D:=VMEMORY-OLD_VMEMORY;
    IF D<0 THEN SIGN:=-1 ELSE SIGN:=1;
    D:=ABS(D);
    IF D<1E-14 THEN D:=1E-14;
    IF D>1E-6 THEN D:=1E-6;
    STP:=D; VPT:=VMEMORY+D*SIGN;
    OLD_VMEMORY:=VMEMORY
  END
  ELSE
  BEGIN
    VPT:=VMEMORY; STP:=VPT/1000
  END
END;

PROCEDURE CHECK_ZEROPOINT(VPT,STP:EXTENDED;VAR ZEROPOINT_YES:BOOLEAN);
(* Checks if the volume resulting from Calc_Vpt() is a minimum or a zeropoint in the
   pressure-volume curve. Therefore we need expression for pressure rather than
   abs(pressure). The last step in calc_vpt was stp*3.
 *)
VAR C_MIN,C_PLUS,MIN,PLUS,PRODUCT:EXTENDED;OLD_ATTRAC_REP:BOOLEAN;
BEGIN
  (* Calc_vpt calculates the minimum in the abs(P(V)) curve. We must now use P(V) to see
     if the solution is a minimum or zeropoint. P(V) is calculated when attrac_rep=true
   *)
  OLD_ATTRAC_REP:=ATTRAC_REP; ATTRAC_REP:=TRUE;
  (* We take a step above and below the solution VPT. If we have a zeropoint then
     [P(V+d)-P]/[P(V-d)-P] is negative. If a minimum it is positive.
   *)
  PLUS:=VPT+3*STP; MIN:=VPT-3*STP;
  C_PLUS:=PRESSURE_EQUATION(PLUS)-PRESSURE;
  C_MIN:=PRESSURE_EQUATION(MIN)-PRESSURE;
  PRODUCT:=C_MIN*C_PLUS;
  IF PRODUCT<=0 THEN
  BEGIN
    (* True zero point in the pressure expression *)
    ZEROPOINT_YES:=TRUE;
    (* Check if bulkmodulus, K>0: stable solution or <0 instable solution: we do that at
       hand of the points (MIN,C_MIN) and (PLUS,C_PLUS) on the P-V curve; if slope is
       >0 we have a negative K=-V*dP/dV
     *)
    IF PLUS>MIN THEN
    BEGIN
      IF C_PLUS>C_MIN THEN ZEROPOINT_YES:=FALSE
    END
    ELSE
    BEGIN
      IF C_MIN>C_PLUS THEN ZEROPOINT_YES:=FALSE
    END
  END
  ELSE ZEROPOINT_YES:=FALSE;
  ATTRAC_REP:=OLD_ATTRAC_REP
END;

PROCEDURE CALC_VPT_MIN(AFTER_ZEROPOINT:BOOLEAN;VAR VPT,STP:EXTENDED);
(* The volume V(P,T) is obtained by minimization of the expression for abs(pressure). *)
VAR VM,STP1,PL,PN,VLAST,MIN_STEP:EXTENDED;
    SW,STOP_IT,LONG_V:BOOLEAN;
    COUNTER:INTEGER;
BEGIN
  (* Set the minimum step in volume *)
  MIN_STEP:=1E-16;
  (* After a zeropoint calculation (after_zeropoint=true) we must take the last volume VPT
     calculated in calc_vpt. When after_zeropoint=false initialize the volume to V0, which is
     the volume at 0 Kelvin, 0 Pascal
   *)
  IF AFTER_ZEROPOINT=FALSE THEN VPT:=V0;
  (* Initialize the step STP in volume in the minimization of abs(pressure) *)
  STP:=VPT/1000; LONG_V:=FALSE; SW:=FALSE; STOP_IT:=FALSE;
  IF AFTER_ZEROPOINT=FALSE THEN ESTIMATE_VPT_STEP(VPT,STP);
  IF ABS(STP)<3*MIN_STEP THEN STP:=3*MIN_STEP; 
  WHILE (ABS(STP)>MIN_STEP) AND (STOP_IT=FALSE) DO
  BEGIN
    STP1:=STP;
    REPEAT
      TROUBLE_VOLUME(VPT,STP); VM:=VPT; COUNTER:=0;
      IF LONG_V=TRUE THEN STP:=STP/3;
      IF (LONG_V=TRUE) AND (ABS(STP)<=ABS(STP1)) THEN LONG_V:=FALSE;
      IF SW=FALSE THEN PL:=PRESSURE_EQUATION(VPT);
      VPT:=VPT+STP; TROUBLE_VOLUME(VPT,STP); SW:=TRUE;
      PN:=PRESSURE_EQUATION(VPT);
      VLAST:=VPT;
      IF PN>=PL THEN
      BEGIN
        STP:=-STP; VPT:=VM; PN:=PL
      END;
      REPEAT
        PL:=PN; VLAST:=VPT; VPT:=VPT+STP; TROUBLE_VOLUME(VPT,STP);
        COUNTER:=COUNTER+1;
        PN:=PRESSURE_EQUATION(VPT);
        EXPAND_STEP(COUNTER,STP,LONG_V);
        (* Limit the search in volume to 10 m3/mol *)
        IF VPT>10 THEN STOP_IT:=TRUE
      UNTIL
        (PN>=PL) OR (STOP_IT=TRUE);
      VPT:=VLAST
    UNTIL
      (LONG_V=FALSE) OR (STOP_IT=TRUE);
    STP:=ABS(STP)/3
  END
END;

PROCEDURE CORRECT_OVERSHOOT(VAR VPT:EXTENDED;VAR ZEROPOINT_YES:BOOLEAN);
(* When zeropoint_yes=false after the first volume calculation, it is possible that the
   step in volume has become too large in the step expansion process. That may result in
   a second zero point of the pressure curve, identical with a second minimum of abs(pressure)
   curve. We try to find the first zero point of the normal pressure back by finding first
   its minimum (if present). When there is a minimum we have 2 solutions for the volume in
   the P-V curve. The first is the stable solution having a positive bulkmodulus (the larger
   volume has a negative bulk modulus). After finding this minimum we search the stable volume
   with CALC_VPT_MIN. We do that by adding a few extra negative steps in volume and subsequently
   scan the minimum in the abs(Pressure) curve again with a small step. After that we check
   if we have a true zero point
 *)
VAR P_OLD,P_NEW,STEP,VPT_OLD:EXTENDED;
    AFTER_ZEROPOINT,OLD_ATTRAC_REP,STOP_IT:BOOLEAN;
BEGIN
  (* Set scan step. Set that the real pressure must be scanned in negative
     volume direction
   *)
  OLD_ATTRAC_REP:=ATTRAC_REP; VPT_OLD:=VPT;
  STEP:=-VPT/1000; ATTRAC_REP:=TRUE; STOP_IT:=FALSE;
  P_OLD:=PRESSURE_EQUATION(VPT);
  REPEAT
    VPT:=VPT+STEP; TROUBLE_VOLUME(VPT,STEP);
    P_NEW:=PRESSURE_EQUATION(VPT);
    IF P_NEW>P_OLD THEN STOP_IT:=TRUE;
    P_OLD:=P_NEW
  UNTIL
    (STOP_IT=TRUE) OR (VPT<=V0/2);
  IF STOP_IT=TRUE THEN
  BEGIN
    (* Few extra steps before the minimum of the real pressure to make sure we are
       on the correct side of it. Step is still negative
     *)
    IF VPT>V0/2 THEN VPT:=VPT+3*STEP;
    (* Set after_zeropoint=true so that ESTIMATE_VPT_STEP is not used in calc_vpt_min().
       Also set attrac_rep=false to scan in the absolute pressure for a minimum
     *)
    AFTER_ZEROPOINT:=TRUE; ATTRAC_REP:=OLD_ATTRAC_REP;
    CALC_VPT_MIN(AFTER_ZEROPOINT,VPT,STEP);
    (* Check if resulting volume is a true zeropoint in the real pressure *)
    CHECK_ZEROPOINT(VPT,STEP,ZEROPOINT_YES);
    (* Reset to incoming volume when not successful *)
    IF ZEROPOINT_YES=FALSE THEN VPT:=VPT_OLD
  END
  ELSE VPT:=VPT_OLD;
  ATTRAC_REP:=OLD_ATTRAC_REP
END;

PROCEDURE CALC_VPT(VAR VPT:EXTENDED);
(* Main procedure to determine the volume. If the resulting minimum in abs(pressure) is not
   a true zeropoint in pressure, we must scan until we are in the vicinity of a new true
   zero point. Only when dP/dV<0, or K>0 a physically realistic solution is achieved.
   When no solutions are available, the substance is mechanically instable: we
   take the volume of the minimum of abs(pressure): that results in dP/dV=0,
   K=0, and is visible in expansivity and therefore in Cp (infinity)
 *)
VAR STP:EXTENDED;
    ZEROPOINT_YES,AFTER_ZEROPOINT:BOOLEAN;
BEGIN
  AFTER_ZEROPOINT:=FALSE;
  (* Calc_vpt must also return its last step: that was 3xstp *)
  CALC_VPT_MIN(AFTER_ZEROPOINT,VPT,STP);
  (* Check if the minimum in the absolute pressure is a zeropoint.
     When bulkmodulus, K<0 then we have an instable solution and we set
     zeropoint_yes=false in CHECK_ZEROPOINT
   *)
  CHECK_ZEROPOINT(VPT,STP,ZEROPOINT_YES);
  (* When zeropoint_yes=false, check if the volume VPT has been reached by overshoot
     in stepping in volume due to the expansion steps in volume in calc_vpt_min.
     In that case VPT maybe the second minimum. CORRECT_OVERSHOOT must correct
     for the volume by searching if a first minimum is present which is a true
     zeropoint in the expression for total pressure (not the abs(pressure))
   *)
  IF ZEROPOINT_YES=FALSE THEN CORRECT_OVERSHOOT(VPT,ZEROPOINT_YES);
  (* VMEMORY contains the last calculated volume.
     Only the first minimum must come in Vmem. That is because in subsequent isothermal
     calculations the minimum may change from a non-zeropoint to a true zeropoint at higher
     pressure. Therefore we must always scan from low volumes, V0 or from VMEMORY
     when the minimum is a zeropoint
   *)
  IF (ZEROPOINT_YES=TRUE) AND (EXTRAPOLATE=TRUE) THEN VMEMORY:=VPT ELSE VMEMORY:=V0;
  (* Landau contribution to volume if defined *)
  IF GET_V0ST=FALSE THEN
  BEGIN
    (* GET_V0ST is only TRUE when the static volume must be determined, which
       is not the case here. The Landau contribution to volume is independent
       from the pressure equation.
       GET_V0ST=TRUE/FALSE in FIX_STATIC_VOLUME of this unit. 
     *)
    IF ABS(LANDAU_YES)>1E-8 THEN VOLUME_LANDAU(1,VPT)
  END
END;

PROCEDURE GET_THERMO_PROPERTIES(VAR VPT,GIBBS,HELMHOLTZ,ENTHALPY,ENERGY,ENTROPY,
                                CP,CV,BULKMOD,DPDT,ALPHA,ZERO_PNT:EXTENDED);
(* Gets thermodynamic properties at (PRESSURE,TEMPERATURE).
   VPT=volume of the substance. ZERO_PNT=zero point vibrational contribution
   to pressure at (0 K, 0 Pa). ALPHA=thermal expansivity. DPDT=isochoric
   temperature derivative of thermal pressure
 *)
VAR DELTA,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L,V_TOTAL,V_LANDAU,A_ELEC,
    U_ELEC,S_ELEC,CV_ELEC,K_ELEC,DPDT_ELEC,A_MAG,U_MAG,S_MAG,CV_MAG,
    GDIS,HDIS,UDIS,SDIS,CPDIS,CVDIS,U_DEF,S_DEF,CV_DEF,K_DEF,DPDT_DEF:EXTENDED;
BEGIN
  DELTA:=1E-08;
  (* Get the volume VPT at pressure,temperature. That volume, VPT, includes the
     contribution due to Landau ordering. We call that V_TOTAL
   *)
  VPT:=V0; CALC_VPT(VPT);
  V_TOTAL:=VPT;
  (* In case of Landau contributions: determine volume without it. That volume
     must be used to determine thermo properties which have nothing to do with
     Landau, such as vibrational properties
   *)
  IF ABS(LANDAU_YES)>DELTA THEN
  BEGIN
    V_LANDAU:=0; VOLUME_LANDAU(1,V_LANDAU);
    (* Now make volume without Landau contribution *)
    VPT:=VPT-V_LANDAU
  END;
  (* Thermal part. Vibrational properties are calculated in unit VIBRATIONS.PAS
   *)
  VIBRATION_PROPERTIES(VPT,ENERGY,ENTROPY,CV,DPDT,HELMHOLTZ,BULKMOD,ZERO_PNT);
  A_ELEC:=0; U_ELEC:=0; S_ELEC:=0; CV_ELEC:=0; K_ELEC:=0; DPDT_ELEC:=0;
  IF ELECTRONIC_YES>0 THEN
  BEGIN
    (* Electronic properties in unit ELECTRONIC.PAS *)
    ELECTRONIC_PROPERTIES(VPT,A_ELEC,U_ELEC,S_ELEC,CV_ELEC,K_ELEC,DPDT_ELEC);
  END;
  A_MAG:=0; U_MAG:=0; S_MAG:=0; CV_MAG:=0;
  IF MAGNETIC_YES>0 THEN
  BEGIN
    (* Magnetic properties. GET_MAGNETIC_PROPERTIES is present in unit MAGNETIC.PAS.
       Presently magnetic properties are not volume dependent
     *)
    GET_MAGNETIC_PROPERTIES(A_MAG,U_MAG,S_MAG,CV_MAG)
  END;
  U_DEF:=0; S_DEF:=0; CV_DEF:=0; K_DEF:=0; DPDT_DEF:=0;
  IF DEFECT_YES>0 THEN
  BEGIN
    (* Mono vacancy defect contribution in unit DEFECTS.PAS *)
    MONOVACANCY(VPT,U_DEF,S_DEF,CV_DEF,K_DEF,DPDT_DEF);
  END;
  ENERGY:=U_REF+ENERGY+STATIC_HELMHOLTZ(VPT)+U_ELEC+U_MAG+U_DEF;
  ENTROPY:=ENTROPY+S_ELEC+S_MAG+S_DEF;
  HELMHOLTZ:=ENERGY-TEMPERATURE*ENTROPY;
  ENTHALPY:=ENERGY+PRESSURE*VPT;
  GIBBS:=ENTHALPY-TEMPERATURE*ENTROPY;
  BULKMOD:=BULKMOD+STATIC_BULKMOD(VPT)+K_ELEC+K_DEF;
  DPDT:=DPDT+DPDT_ELEC+DPDT_DEF;
  ALPHA:=DPDT/BULKMOD;
  CV:=CV+CV_ELEC+CV_MAG+CV_DEF;
  CP:=CV+ALPHA*ALPHA*BULKMOD*VPT*TEMPERATURE;
  (* Check if Landau theory must be applied for endmember K in phase P *)
  IF ABS(LANDAU_YES)>DELTA THEN
  BEGIN
    (* Add Landau contributions when LANDAU_YES>0. We have determined all
       properties without Landau with the volume, VPT, that does not include
       Landau ordering. Now we must set the volume, VPT, to the total volume
       including Landau ordering, V_TOTAL
     *)
    VPT:=V_TOTAL;
    LANDAU_CALC(1,VPT,GIBBS,ENTHALPY,ENERGY,ENTROPY,CV,CP,ALPHA,BULKMOD,
                DPDT,VL,GL,HL,UL,SL,CV_L,CP_L,AL_L,KT_L,DPDT_L);
    (* Helmholtz energy is not returned from LANDAU_CALC *)
    HELMHOLTZ:=ENERGY-TEMPERATURE*ENTROPY
  END;
  (* Contribution due to cation disorder *)
  IF CATION_DISORDER_YES>DELTA THEN
  BEGIN
    GDIS:=0; HDIS:=0; UDIS:=0; SDIS:=0; CPDIS:=0; CVDIS:=0;
    DISORDER_CALC(TEMPERATURE,GDIS,HDIS,UDIS,SDIS,CPDIS,CVDIS);
    GIBBS:=GIBBS+GDIS;
    ENERGY:=ENERGY+UDIS;
    ENTROPY:=ENTROPY+SDIS;
    ENTHALPY:=ENTHALPY+HDIS;
    CV:=CV+CVDIS;
    CP:=CP+CPDIS
  END
END;

PROCEDURE FIX_STATIC_VOLUME(VAR V0_ST_FIX:EXTENDED;VAR FIX_YES:BOOLEAN);
(* When you change the input parameters in the input file yourself, the lattice becomes
   unstable. The EoS pressure will not match the external pressure at (0 K, 0 Pa).
   That is remedied by changing the static lattice volume
 *)
VAR VPT,DIFFERENCE,OLD_TEMPERATURE,OLD_PRESSURE:EXTENDED;
BEGIN
  OLD_PRESSURE:=PRESSURE; OLD_TEMPERATURE:=TEMPERATURE;
  PRESSURE:=0; TEMPERATURE:=0; VPT:=V0; FIX_YES:=FALSE;
  (* Determine thermal pressure at V0. The electronic pressure is zero.
     Thermal pressure is a constant.
     Next determine the volume ratio = compression in the static part of
     the pressure
   *)
  P_ZEROVIB:=THERMAL_PRESSURE(VPT);
  (* Set GET_V0ST=TRUE to indicate to PRESSURE_EQUATION that P_ZEROVIB must
     be used and not THERMAL_PRESSURE. Otherwise that function is scanned in
     volume whereas we need V0 as solution
   *)
  GET_V0ST:=TRUE;
  (* Get the volume *)
  CALC_VPT(VPT);
  (* We calculated VPT at 0 K, 0 Pa, from which it follws that:
     VPT/V0_st(old)=V0(new)/V0_st(old). We want to keep the value for V0.
     Therefore we must set V0(new)/V0_st(old)=V0(old)/V0_st(new)
     giving V0_st(new)=V0(old)*V0_st(old)/V0(new)
   *)
  V0_ST_FIX:=V0*V0_ST/VPT;
  (* The difference between the new V0_st and old V0_st in cm3/mol must be
     smaller than a critical value, 1e-06 cm3/mol. If not, the user has changed
     input parameters justifying a change in static lattice volume
   *)
  DIFFERENCE:=(V0_ST_FIX-V0_ST)*1E+06;
  IF ABS(DIFFERENCE)>1E-06 THEN
  BEGIN
    FIX_YES:=TRUE
  END;
  (* Reset GET_V0ST=FALSE, its standard value, reset temperature, pressure *)
  GET_V0ST:=FALSE; PRESSURE:=OLD_PRESSURE; TEMPERATURE:=OLD_TEMPERATURE
END;

END.
