PROGRAM me;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of program 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   --------------------------------------------------------------------------

   This program is based on the multiple Einstein method described by:
   Jacobs MHG, Schmid-Fetzer R, van den Berg AP (2013) An alternative use of
   Kieffer's lattice dynamics model using vibrational density of states for
   constructing thermodynamic database. Phys Chem Minerals, 40:207-227

   ### Version 9: July 1st, 2021 ###

   This program is based on simplified functions and procedures in FreePascal 
   units of program XiPT. Parts of output files with extension '.mef'
   generated by XiPT serve as input in this program 'me.pas'
   Some features of XiPT are given by:
   - Jacobs MHG, Oonk HAJ (2006) Calphad 30:185-190
   - Jacobs MHG, Oonk HAJ (2012) Equilibrium between phases of matter.
     Supplemental text for material science and high-pressure geophysics.
     Springer, Dordrecht
   ----------------------------------------------------------------------------

   - This program allows calculating thermodynamic properties including
     shear modulus for pure solid substances (not solid solution phases).

   - Installing Multiple-Einstein program for running in Ubuntu:
     A. Install FreePascal in Linux:
        1. Open a terminal window by pressing simultaneously:
           cntrl-alt-t
        2. At the command prompt in the terminal window type:
           sudo apt-get install fp-compiler
     B. Install ME program:
        3. Open a terminal: cntrl-alt-t
        4. Unpack me.tgz by typing:
           tar -zxf me.tgz
        4. Compile the ME program by typing:
           fpc me.pas
        5. Run the program by typing:
           ./me test.inp

   - Installing Multiple-Einstein program for running in Windows:
     A. Install FreePascal in Windows:
        1. Download FreePascal from website https://www.freepascal.org
           fpc-3.0.4.i386-win32.cross.x86_64-win64.exe and
           fpc-3.0.4.i386-win32.exe
        2. Execute both programs.
     B. Install ME program:
        3. Open a powershell window (windows10) or command prompt (windows7)
        4. unzip me.zip by typing:
           unzip me.zip
        5. Compile the ME program by typing:
           fpc me.pas
        6. Run the program by typing:
          .\me test.inp

   Features:
   - Frequency/Gruneisen expressions that can be treated are given by:
       1. Al'tshuler LV, Brusnikin SE, Kuz'menkov EA (1987) Isotherms and Gruneisen
          functions for 25 metals. J Appl Mech Tech Phys 28:129-141
       2. Stixrude L, Lithgow-Bertelloni (2005) Thermodynamics of mantle minerals
          - I. Physical properties. Geophys J Int 162:610-632
   - Equations of State,                                                  [version>=1]
       1. Up to 8-th order Birch-Murnaghan Equation of State
       2. 3rd order Vinet Equation of State
       3. Keane's (1954) Equation of state                                [version>=9]
       4. Qin et al (2008) Equation of State                              [version>=9]
   - Free electron gas electronic contribution,                           [version>=1]
   - Landau formalism,                                                    [version>=2]
       1. 2nd order transitions, e.g. stishovite
       2. Tricritical transitions, e.g quartz
       3. 1st order transitions
   - Magnetic contributions temperature dependent (not volume dependent), [version>=2]
   - Crystal Field electronic contribution,                               [version>=3]
   - generating GnuPlot files for plotting vdos                           [version>=4]
   - making 'me' input file from experimental/abinitio VDoS with 'medos'  [program medos]
   - Cation disorder                                                      [version>=5]
   - Mono vacancy defects                                                 [version>=6]
   - 'me' runs in Linux and in powershell of windows                      [version>=7]
   - Hugoniot, isentrope, clone VDoS                                      [version>=8]

   To do: - elastic constants
          - solid solutions
          - spin transition
          - magnetic contributions to thermodynamic properties (volume dependent)
          - optimizer
          - ...
 *)

USES CRT,sysutils,variables,readinput,maketable,plotvdos,clonevdos;


PROCEDURE MAIN_ME;
(* Main procedure *)
VAR IT_EXISTS:BOOLEAN;
BEGIN
  LINUX:=TRUE; WHAT_SLASH:='/'; LINUX_OR_WINDOWS;
  IF PARAMCOUNT=1 THEN
  BEGIN
    (* Check if the input file with model parameters is present.
       Procedure 'EXIST_FILE' is present in unit READINPUT.PAS
     *)
    NAME_INPUTFILE:=PARAMSTR(1);
    IT_EXISTS:=FALSE; IT_EXISTS:=EXIST_FILE(NAME_INPUTFILE);
    IF IT_EXISTS=TRUE THEN
    BEGIN
      (* Input file exists. Read the input in unit READINPUT.PAS *)
      INPUT_OKAY:=FALSE; READ_INPUT_FILE(INPUT_OKAY);
      IF INPUT_OKAY=TRUE THEN
      BEGIN
        CASE CALCULATION_TYPE OF
      1,2,4,5:BEGIN
                (* Carry out the calculation in unit MAKETABLE.PAS *)
                MAKE_TABLE
              END;
            3:BEGIN
                (* Make a VDoS at (PRESSURE, TEMPERATURE) *)
                MAKE_VDOS
              END;
            6:BEGIN
                (* Clone the VDoS to a VDoS with a smaller number of
                   Einstein frequencies
                 *)
                CONVERT_VDOS
              END
        END
      END
    END
    ELSE
    BEGIN
      (* Input file does not exist: terminate program *)
      WRITELN; WRITELN;
      WRITELN('----------------------------');
      WRITELN('Input file does not exist...');
      WRITELN('----------------------------');
      WRITELN; WRITELN
    END
  END
  ELSE
  BEGIN
    (* If no input file as argument is defined then PARAMCOUNT=0 *)
    WRITELN; WRITELN;
    WRITELN('-------------------------------------------------------------');
    WRITELN('Use this program in combination with an input file by typing:');
    IF LINUX=TRUE THEN WRITELN('./me inputfile.mef')
    ELSE WRITELN('.\me inputfile.mef');
    WRITELN('next press enter');
    WRITELN('-------------------------------------------------------------');
    WRITELN; WRITELN
  END
END;

BEGIN
  MAIN_ME
END.
