UNIT plotvdos;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Unit for making a plot of the VDoS in the form of Einstein contiunua.
   'me' puts the wavenumber in (cm^-1, THz or meV) on the X-axis and the VDoS
   value on the Y-axis.
   See for a plot Jacobs&deJong (2005), Phys. Chem. Minerals, 32, 614-626.
   Einstein temperatures will be expressed in cm^-1, THz or meV
   Example:
   Defining w as the wavenumber and X=h*nu/kT=h*c*w/kT, with w in m^-1
   it follows that: X=100*h*c*w/kT when w is in cm^-1.
                    g(X)dX=g(w)dw
       (100*h*c/kT)*g(X)dw=g(w)dw
                      g(w)=(100*h*c/kT)*g(X)
   The calculation part uses X for calculating g(X)/Na with Na Avogadro's number.
   Because wavenumbers are on the X-axis we put g(w) [unit cm] on the Y-axis and
   the resulting g(X) must be multiplied with 100*h*c/kT.
   Values of G(X) are of the order 6 and values for g(w) of the order 0.03 for MgO.
   On Y-axis we put the label g(w) [cm] when on the X-axis is cm^-1.
   The multiplication factor is incorporated in OC_DOS() which calculates VDoS

   To convert the calculations from X=h*nu/kT to the units below:
   --------------------------------------------------------------
                 X->wavenr(cm-1)   X->nu(THz)      X->E(meV)
   UNIT_CAL_X    T/HCK             1E-12*k*T/h     1E+3*kT/q
   UNIT_CAL_Y    HCK/T             1E+12*h/kT      q/1E+3/kT
   To convert the experiments from the unit specified in the .exp file to the
   desired unit above:
                 UNIT_EXP_X      UNIT_EXP_Y
   cm-1->cm-1    1               1
       ->THz     100*c*1E-12     1
       ->meV     h*c*100/q/1000  1
   THz ->cm-1    1E+12/c/100     1
       ->THz     1               1
       ->meV     1E+12*h*1000/q  1
   meV ->cm-1    1E-3*1E-2/q/h/c 1
       ->THz     1E-3*1E-12/q/h  1
       ->meV     1               1
   For UNIT_EXP_Y we take 1/UNIT_EXP_X because the surface under the VDOS is
   dimensionless
 *)
INTERFACE
USES CRT,sysutils,variables,readinput,makevolume;


PROCEDURE MAKE_VDOS;
IMPLEMENTATION

TYPE RY_TWICE_FREQUENCY=ARRAY[1..2*NUMBER_OF_FREQUENCIES] OF EXTENDED;
VAR F_DAT,F_GRF:TEXT;                  (* F_GRF: script file for GnuPlot
                                          F_DAT: datafile (VDoS points)
                                        *)
    LAB_X,LAB_Y:ANSISTRING;            (* LAB_X: label on X-axis
                                          LAB_Y: label on Y-axis
                                        *)
    XN,X_IN:RY_TWICE_FREQUENCY;        (* Frequencies of continua in the VDoS *)
    FRAC_N:VIBRATIONAL;                (* Fractions of continua *)
    X0_DUMMY,X_DUMMY:RY_TWICE_FREQUENCY;
                                       (* Theta0/T and theta/T when T>0 *)
    X_MAX,X_MIN,Y_MAX,Y_MIN,X_MIN_SET,X_MAX_SET,Y_MIN_SET,Y_MAX_SET,
    UNIT_CAL_X,UNIT_CAL_Y,UNIT_EXP_X,UNIT_EXP_Y,V0_ST_ORIGINAL,V0_ST_FIX,VPT:EXTENDED;
                                       (* X_MAX,Y_MAX: maximum (X,Y) calculated VDoS
                                          X_MIN,Y_MIN: minimum (X,Y) calculated VDoS
                                          X_MAX_SET,Y_MAX_SET: preset maximum VDoS
                                          X_MIN_SET,Y_MIN_SET: preset minimum VDoS
                                          UNIT_CAL_X: factor for converting X=h*nu/kT
                                                      to desired unit on X-axis
                                                      [calculated VDoS]
                                          UNIT_CAL_Y: 1/UNIT_EXP_X
                                          UNIT_EXP_X: factor for converting experimental
                                                      frequency unit to desired unit
                                          UNIT_EXP_Y: 1/UNIT_EXP_X
                                          V0_ST_ORIGINAL: uncorrected static volume that
                                            was read from the 'me' inputfile
                                          V0_ST_FIX: corrected static volume [when model-
                                            parameters are changed by user]
                                          VPT: volume calculated at [PRESSURE,TEMPERATURE]
                                        *)
    FIX_YES:BOOLEAN;                   (* FIX_YES: True when static volume must be
                                            corrected
                                        *)


PROCEDURE SETOUTL(TESTNAME:ANSISTRING);
(* Set file open that must contain the experimental VDoS data *)
BEGIN
  ASSIGN(F_DAT,TESTNAME);REWRITE(F_DAT)
END;

PROCEDURE SETOUTGRF(TESTNAME:ANSISTRING);
(* Set script file for GnuPlot open *)
BEGIN
  ASSIGN(F_GRF,TESTNAME);REWRITE(F_GRF)
END;

PROCEDURE GET_PATH_INPUTFILE(VAR NAME_INPUTFILE:ANSISTRING);
(* Gets complete name of input file, NAME_INPUTFILE, including its path *)
VAR PATH:ANSISTRING;
    J,LENG:INTEGER;
    HAS_SLASH:BOOLEAN;
BEGIN
  (* Check if name of the input file has a path or not *)
  LENG:=LENGTH(NAME_INPUTFILE); HAS_SLASH:=FALSE;
  IF LENG>0 THEN
  BEGIN
    FOR J:=1 TO LENG DO
      IF NAME_INPUTFILE[J]=WHAT_SLASH THEN HAS_SLASH:=TRUE
  END;
  (* When forward or back slash present, HAS_SLASH=TRUE, do not change the name
     of the input file. When FALSE assume path is the same as program
   *)
  IF HAS_SLASH=FALSE THEN
  BEGIN
    (* No forward or back slash: input file has been called from current directory
     *)
    PATH:=GETCURRENTDIR;
    NAME_INPUTFILE:=PATH+WHAT_SLASH+NAME_INPUTFILE
  END
END;

PROCEDURE REPORT_STATICVOLUME_FIX;
(* Report if static volume has been fixed. This is the case when model parameters
   were changed by the user
 *)
VAR DIFFERENCE,RELATIVE_DIFFERENCE:EXTENDED;
    J:INTEGER;
BEGIN
  IF FIX_YES=TRUE THEN
  BEGIN
    (* You changed input parameters. FIX_STATIC_VOLUME() in unit MAKEVOLUME.PAS
       changed the static volume
     *)
    FOR J:=1 TO 80 DO WRITE(F_GRF,'#');
    WRITELN(F_GRF);
    WRITE(F_GRF,'# Model parameters were changed or preset in the input file. Therefore the total');
    WRITELN(F_GRF);
    WRITELN(F_GRF,'# equation of state pressure, P_total(T=0,V0), does not match the external');
    WRITE(F_GRF,'# pressure at (0 K, 0 Pa).');
    WRITELN(F_GRF);
    WRITE(F_GRF,'# ''me'' repaired this by changing the static volume. The input parameters now give');
    WRITELN(F_GRF);
    WRITE(F_GRF,'# P_total(T=0,V0) = 0 when the external pressure is zero.');
    WRITELN(F_GRF); WRITELN(F_GRF);
    WRITELN(F_GRF,'# Old static volume: ',V0_ST_ORIGINAL*1E+06:10:6,' cm3/mol');
    WRITELN(F_GRF,'# New static volume: ',V0_ST_FIX*1E+06:10:6,' cm3/mol');
    DIFFERENCE:=V0_ST_FIX-V0_ST_ORIGINAL;
    RELATIVE_DIFFERENCE:=100*DIFFERENCE/V0_ST_ORIGINAL;
    WRITE(F_GRF,'# Difference       : ',DIFFERENCE*1E+06:10:6,' cm3/mol');
    WRITELN(F_GRF,'   [',RELATIVE_DIFFERENCE:10:6,'%]');
    FOR J:=1 TO 80 DO WRITE(F_GRF,'#');
    WRITELN(F_GRF)
  END
END;

PROCEDURE HEAD_SCRIPT;
(* Makes the header of the Gnuplot script file *)
VAR MONTH_NAME:STR14;
    YEAR,DAY,VDOS_STR:STR4;
    HOUR,MINUTE,SECOND:STR2;
BEGIN
  WRITELN(F_GRF,'# ---------------------------------------------------------------------------');
  (* Name of the program that constructed the output file *)
  WRITELN(F_GRF,'# This file calculated by: me',ME_VERSION);
  (* Get path+name of input file *)
  GET_PATH_INPUTFILE(NAME_INPUTFILE);
  WRITELN(F_GRF,'# Input file    : ',NAME_INPUTFILE);
  (* Date-Time this file is constructed *)
  CURRENT_DATE_TIME(YEAR,MONTH_NAME,DAY,HOUR,MINUTE,SECOND);
  WRITE(F_GRF,'# Constructed on: ',MONTH_NAME,' ',DAY,', ',YEAR);
  WRITELN(F_GRF,' / ',HOUR,'hr:',MINUTE,'min:',SECOND,'sec');
  (* Some explanations on units *)
  WRITELN(F_GRF,'#');
  WRITELN(F_GRF,'# Script file for Gnuplot: Vibrational Density of States');
  WRITE(F_GRF,'# The resulting VDoS values are g(');
  CASE VDOS_UNIT_DESIRED OF
    1:VDOS_STR:='cm-1';
    2:VDOS_STR:='THz';
    3:VDOS_STR:='meV'
  END;
  CASE VDOS_UNIT_DESIRED OF
    1:WRITE(F_GRF,'w');
    2:WRITE(F_GRF,'f');
    3:WRITE(F_GRF,'E')
  END;
  WRITE(F_GRF,')/Na in 1/',VDOS_STR,', where g(');
  CASE VDOS_UNIT_DESIRED OF
    1:WRITE(F_GRF,'w');
    2:WRITE(F_GRF,'f');
    3:WRITE(F_GRF,'E')
  END;
  WRITELN(F_GRF,') is the');
  WRITE(F_GRF,'# ');
  CASE VDOS_UNIT_DESIRED OF
    1:WRITE(F_GRF,'wavenumber');
    2:WRITE(F_GRF,'normal frequency');
    3:WRITE(F_GRF,'energy')
  END;
  WRITE(F_GRF,' VDoS, and ');
  CASE VDOS_UNIT_DESIRED OF
    1:WRITE(F_GRF,'w');
    2:WRITE(F_GRF,'f');
    3:WRITE(F_GRF,'E')
  END;
  WRITE(F_GRF,' in ',VDOS_STR,'.');
  WRITELN(F_GRF,' Na is Avogadros number.');
  WRITE(F_GRF,'# Experiments: ');
  IF NAME_VDOSFILE='' THEN WRITELN(F_GRF,'No file')
  ELSE WRITELN(F_GRF,NAME_VDOSFILE);
  WRITELN(F_GRF,'# See end of this file for conversion to *.eps file for use in GIMP');
  (* Write PRESSURE, TEMPERATURE and calculated volume *)
  WRITELN(F_GRF,'#');
  WRITELN(F_GRF,'# Pressure    = ',PRESSURE:15,' Pa');
  WRITELN(F_GRF,'# Temperature = ',TEMPERATURE:15,' K');
  WRITELN(F_GRF,'# Volume      = ',VPT*1E+06:15,' cm3/mol');
  WRITELN(F_GRF,'#');
  (* Report if static volume has been fixed *)
  REPORT_STATICVOLUME_FIX;
  WRITELN(F_GRF,'# ---------------------------------------------------------------------------');
  WRITELN(F_GRF,'#');
  (* Terminal must be set before multiplot: it is not necessary to
     enter the next 4 lines. In that case gnuplot selects its
     standard terminal
   *)
  WRITELN(F_GRF,'# Set terminal to wxt: comment this line when plotting to file with extension .eps');
  WRITELN(F_GRF,'set terminal wxt');
  WRITELN(F_GRF,'# Set font to Arial 12: comment this line when plotting to file with extension .eps');
  WRITELN(F_GRF,'set term wxt font "Arial, 12"');
  WRITELN(F_GRF,'# Set frame size of total plot: plot including legends');
  WRITELN(F_GRF,'set size 1,1');
  WRITELN(F_GRF,'# set origin of the plot ');
  WRITELN(F_GRF,'set origin 0,0');
  WRITELN(F_GRF,'# Set minimum/maximum variables in X-Y plot');
  X_MIN_SET:=X_MIN; X_MAX_SET:=X_MAX; Y_MIN_SET:=Y_MIN; Y_MAX_SET:=Y_MAX;
  WRITELN(F_GRF,'Xmin=',X_MIN_SET:15);
  WRITELN(F_GRF,'Xmax=',X_MAX_SET:15);
  WRITELN(F_GRF,'Ymin=',Y_MIN_SET:15);
  WRITELN(F_GRF,'Ymax=',Y_MAX_SET:15);
  WRITELN(F_GRF,'# Set Multiplot to plot several plots in one figure');
  WRITELN(F_GRF,'set multiplot');
  WRITELN(F_GRF,'# Set plot size and origin in the host plot (1st plot)');
  WRITELN(F_GRF,'# Make X-axis shorter in host plot for including legends later');
  WRITELN(F_GRF,'set size 0.8,1');
  WRITELN(F_GRF,'set origin 0,0');
  WRITELN(F_GRF,'# Set axes tic marks on the host plot independent of 2nd plot (nomirror)');
  WRITELN(F_GRF,'# set xtics nomirror');
  WRITELN(F_GRF,'# set ytics nomirror');
  WRITELN(F_GRF,'# Use minimum/maximum values declared above for X-Y plot');
  WRITELN(F_GRF,'set xrange [Xmin:Xmax]');
  WRITELN(F_GRF,'set yrange [Ymin:Ymax]');
  WRITELN(F_GRF,'# Labels for x and y axis ');
  CASE VDOS_UNIT_DESIRED OF
    1:BEGIN
        LAB_X:='Wavenumber, 1/cm';
        LAB_Y:='g(w)/Na, cm'
      END;
    2:BEGIN
        LAB_X:='Frequency, THz';
        LAB_Y:='g(f)/Na, 1/THz'
      END;
    3:BEGIN
        LAB_X:='Energy, meV';
        LAB_Y:='g(E)/Na, 1/meV'
      END
  END;
  WRITELN(F_GRF,'set xlabel "',LAB_X,'"');
  WRITELN(F_GRF,'set ylabel "',LAB_Y,'"');
  WRITELN(F_GRF,'# Remove the standard legend label');
  WRITELN(F_GRF,'unset key');
  WRITELN(F_GRF,'#');
  WRITELN(F_GRF,'# Plot a (smooth csplines)/lines/dots through data in .res file');
END;

PROCEDURE ADD_FILE_SCRIPT(FIL:ANSISTRING;SPLINE_IT,COLOR:INTEGER;SYMBOL:ANSISTRING);
(* Add the .dat file to script. 
   SPLINE_IT=0: If experimental data must be plotted
   SPLINE_IT=1: If the points in the file must be connected with lines
   SPLINE_IT=2: If the file must be splined 
 *)
VAR S:ANSISTRING;PT:INTEGER;
BEGIN
  IF SPLINE_IT=2 THEN SPLINE_IT:=1;
  WRITE(F_GRF,'plot '); WRITE(F_GRF,'''');
  WRITE(F_GRF,FIL); WRITE(F_GRF,'''');
  WRITE(F_GRF,' using 1:2 with');
  IF SPLINE_IT=0 THEN
  BEGIN
    WRITE(F_GRF,' points pt');
    (* pt gives the point_type=square, diamond etc *)
    PT:=1;
    IF SYMBOL='+' THEN PT:=1
    ELSE
      IF SYMBOL='x' THEN PT:=2
      ELSE
        IF SYMBOL='*' THEN PT:=3
        ELSE
          IF SYMBOL='Square' THEN PT:=4
          ELSE
            IF SYMBOL='Square_f' THEN PT:=5
            ELSE
              IF SYMBOL='Circle' THEN PT:=6
              ELSE
                IF (SYMBOL='Circle_f') OR (SYMBOL='Star') THEN PT:=7
                ELSE
                  IF SYMBOL='Triangle' THEN PT:=8
                  ELSE
                    IF SYMBOL='Triangle_f' THEN PT:=9
                    ELSE
                      IF SYMBOL='Triangle_i' THEN PT:=10
                      ELSE
                        IF (SYMBOL='Triangle_i_f') OR (SYMBOL='Triangle_f_i') THEN PT:=11
                        ELSE
                          IF SYMBOL='Diamond' THEN PT:=12
                          ELSE
                            IF SYMBOL='Diamond_f' THEN PT:=13;
    WRITE(F_GRF,' ',PT);
    (* the color *)
    CASE COLOR OF
      0:S:='black';
      1:S:='dark-blue';
      2:S:='dark-green';
      3:S:='dark-cyan';
      4:S:='dark-red';
      5:S:='dark-magenta';
      6:S:='dark-orange';
      7:S:='grey';
      8:S:='dark-grey';
      9:S:='blue';
      10:S:='green';
      11:S:='cyan';
      12:S:='red';
      13:S:='magenta';
      14:S:='dark-yellow';
      15:S:='orange'
    END;
    (* ps means point_size and lc the line color *)
    WRITE(F_GRF,' ps 1 lc rgb')
  END;
  IF SPLINE_IT=1 THEN
  BEGIN
    WRITE(F_GRF,' lines');
    WRITE(F_GRF,' lw 3 lc rgb')
  END;
  IF SPLINE_IT=2 THEN
  BEGIN
    WRITE(F_GRF,' lines');
    WRITE(F_GRF,' smooth csplines');
    WRITE(F_GRF,' lw 1 lc rgb')
  END;
  IF (SPLINE_IT=1) OR (SPLINE_IT=2) THEN
  BEGIN
    CASE COLOR OF
      0:S:='black';
      1:S:='dark-blue';
      2:S:='dark-green';
      3:S:='dark-cyan';
      4:S:='dark-red';
      5:S:='dark-magenta';
      6:S:='dark-orange';
      7:S:='grey';
      8:S:='dark-grey';
      9:S:='blue';
      10:S:='green';
      11:S:='cyan';
      12:S:='red';
      13:S:='magenta';
      14:S:='dark-yellow';
      15:S:='orange'
    END
  END;
  WRITELN(F_GRF,' "',S,'"')
END;

PROCEDURE FINALIZE_SCRIPT(NAME:ANSISTRING);
(* Gives recommendations to make a high-resolution plot in program GIMP *)
VAR J:INTEGER;
BEGIN
  FOR J:=1 TO 79 DO WRITE(F_GRF,'#');
  WRITELN(F_GRF,'#');
  WRITELN(F_GRF,'# Recommendation: To make a high-resolution plot from this .grf file in GIMP');
  WRITELN(F_GRF,'# replace the lines on top of this file: set terminal wxt');
  WRITELN(F_GRF,'#                                        set term wxt font "Arial,12"');
  WRITELN(F_GRF,'# by:');
  WRITELN(F_GRF,'# set terminal post enh eps color solid "Arial" 20');
  WRITELN(F_GRF,'# set output "',NAME,'.eps"');
  WRITELN(F_GRF,'# In GIMP image editor increase resolution to desired value');
  WRITELN(F_GRF,'# and save the resulting .eps file in GIMP.');
END;

PROCEDURE MAXMIN;
(* Get maximum and minimum of experimental data: needed for head_script. *)
VAR X_MIN_EXP,X_MAX_EXP,Y_MIN_EXP,Y_MAX_EXP:EXTENDED;
    J:INTEGER;
BEGIN
  X_MIN_EXP:=1E+30; X_MAX_EXP:=-1E+30; Y_MIN_EXP:=1E+30; Y_MAX_EXP:=-1E+30;
  (* Minimum and maximum in experimental values *)
  FOR J:=1 TO NUM_VDOS_POINTS DO
  BEGIN
    IF X_EXP[J]>X_MAX_EXP THEN X_MAX_EXP:=X_EXP[J];
    IF X_EXP[J]<X_MIN_EXP THEN X_MIN_EXP:=X_EXP[J];
    IF Y_EXP[J]>Y_MAX_EXP THEN Y_MAX_EXP:=Y_EXP[J];
    IF Y_EXP[J]<Y_MIN_EXP THEN Y_MIN_EXP:=Y_EXP[J]
  END;
  (* If minimum/maxmimum smaller/larger than calculated VDoS take them as min/max *)
  IF X_MIN_EXP<X_MIN THEN X_MIN:=X_MIN_EXP;
  IF X_MAX_EXP>X_MAX THEN X_MAX:=X_MAX_EXP;
  IF Y_MIN_EXP<Y_MIN THEN Y_MIN:=Y_MIN_EXP;
  IF Y_MAX_EXP>Y_MAX THEN Y_MAX:=Y_MAX_EXP
END;

PROCEDURE SORT_OC_WIDTH;
(* Sort continua on width. Continuum with largest width comes first. We sort
   on frequencies of the optic continua (OC) and their fractions
 *)
VAR WIDTH_MAX,WIDTH,W_HELP:EXTENDED;
    J,I_MAX,J_MAX,MAX:INTEGER;
    LARGEST_FOUND:BOOLEAN;
BEGIN
  IF N_EINSTEIN>1 THEN
  BEGIN
    (* First (left) cut_off frequency of the first OC is in XN[1] and last
       (right) cut-off frequency has XN[2*N_EINSTEIN]. I_MIN and J denote the
       OC number. The first cut_off frequency of OC is given by:
       freq_left=2+2*OC-3
     *)
    I_MAX:=1; MAX:=N_EINSTEIN;
    REPEAT
      WIDTH_MAX:=XN[2*I_MAX]-XN[-1+2*I_MAX]; LARGEST_FOUND:=FALSE;
      FOR J:=I_MAX+1 TO MAX DO
      BEGIN
        (* Determine lowest width. The first cut_off frequency is given by
           freq_left=2+2*OC-3=2+2*J-3=2*J-1
         *)
        WIDTH:=XN[2*J]-XN[-1+2*J];
        IF WIDTH>WIDTH_MAX THEN
        BEGIN
          WIDTH_MAX:=WIDTH; J_MAX:=J; LARGEST_FOUND:=TRUE
        END
      END;
      (* Store the lowest width: interchange the continua and fractions *)
      IF LARGEST_FOUND=TRUE THEN
      BEGIN
        W_HELP:=XN[-1+2*I_MAX]; XN[-1+2*I_MAX]:=XN[-1+2*J_MAX];
        XN[-1+2*J_MAX]:=W_HELP;
        W_HELP:=XN[2*I_MAX]; XN[2*I_MAX]:=XN[2*J_MAX];
        XN[2*J_MAX]:=W_HELP;
        W_HELP:=FRAC_N[I_MAX]; FRAC_N[I_MAX]:=FRAC_N[J_MAX]; FRAC_N[J_MAX]:=W_HELP
      END;
      (* In the nex cycle we must not consider the lowest width OC in I_MIN *)
      I_MAX:=I_MAX+1
    UNTIL
      I_MAX>MAX-1
  END;
END;

FUNCTION OC_DOS(N_OC:INTEGER;X:EXTENDED):EXTENDED;
(* Calculates Optical Continua VDOS function.
   That is done for OC=1 to N_OC. The frequencies were already prepared and
   are in XN[1..2*OC] and the fractions are in FRAC[1..OC]. The frequencies
   are as h*nu/kT when T>1 and h*nu/k when T<=1. Multiply OC_DOS with
   HCK/T to arrive at the wavenumber VDoS g(w), cm: see header of
   this unit
 *)
VAR IK,J:INTEGER;
    FAC,SUMZ,SUM,Z1,Z2:EXTENDED;
BEGIN
  FAC:=3*N_MOL;
  IK:=1; SUM:=0; SUMZ:=0;
  (* Left cut-off frequency of mode N_OC is -1+2*N_OC with fraction frac[N_OC] *)
  FOR J:=1 TO N_OC DO
  BEGIN
    IK:=IK+1;
    Z2:=XN[IK];
    Z1:=XN[IK-1];
    IF (X>Z1) AND (X<Z2) THEN SUM:=SUM+FRAC_N[J]/(Z2-Z1);
    IK:=IK+1
  END;
  SUMZ:=SUMZ+FAC*SUM;
  (* Convert to wavenumber VDoS: see header of this unit. When to cm
     that would be SUMZ*HCK/T (T>1) or SUMZ*HCK (T<=1):
     UNIT_CAL_Y=HCK
   *)
  IF TEMPERATURE>1 THEN SUMZ:=SUMZ*UNIT_CAL_Y/TEMPERATURE ELSE SUMZ:=SUMZ*UNIT_CAL_Y;
  OC_DOS:=SUMZ
END;

PROCEDURE GET_MAX_FREQ;
(* Gets the maximum frequency from XN and converts it to wave numbers
   Gets the maximum value of the VDoS
 *)
VAR FRAC_MAX,MAX,WAVE,XF_L,XF_U,XF,YF:EXTENDED;
    J,JMAX:INTEGER;
BEGIN
  MAX:=-1;
  (* Get maximum frequency *)
  FOR J:=1 TO 2*N_EINSTEIN DO
  BEGIN
    IF XN[J]>MAX THEN MAX:=XN[J]
  END;
  (* When T>1 then XN=h*nu/kT=h*c*wave/kT=HCK*wave/T 
     else h*nu/k=h*c*wave/k=HCK*wave and wave in cm-1.
     unit_cal_x=1/HCK
   *)
  IF TEMPERATURE>1 THEN WAVE:=MAX*TEMPERATURE*UNIT_CAL_X ELSE WAVE:=MAX*UNIT_CAL_X;
  X_MAX:=WAVE;
  (* Get maximum fraction for maximum value in the VDoS *)
  FRAC_MAX:=0; JMAX:=1;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    IF FRAC_N[J]>FRAC_MAX THEN
    BEGIN
      FRAC_MAX:=FRAC_N[J]; JMAX:=J
    END
  END;
  (* Maximum value of vdos. Determine lower and upper cut-off frequencies of
     the JMAX continuum and take the value in the middle
   *)
  XF_L:=XN[2*JMAX-1]; XF_U:=XN[2*JMAX]; 
  XF:=(XF_L+XF_U)/2; YF:=OC_DOS(N_EINSTEIN,XF);
  IF YF>Y_MAX THEN Y_MAX:=YF
END;

PROCEDURE STRING_NUMBER(X:EXTENDED;VAR S:ANSISTRING);
(* Makes a string of a number without blanks *)
VAR STRIN:ANSISTRING;
    J,LEN:INTEGER;
BEGIN
  IF ABS(X)>9.9E+04 THEN STR(X:15,S) ELSE STR(X:15:6,S);
  LEN:=LENGTH(S); J:=1; STRIN:='';
  IF LEN>0 THEN
  BEGIN
    REPEAT
      IF J<=LEN THEN
      BEGIN
        IF S[J]<>' ' THEN STRIN:=STRIN+S[J]
      END;
      J:=J+1
    UNTIL
      J>LEN;
    S:=STRIN
  END
END;

PROCEDURE MAKE_OBJ_BOX(OBJECT_NR,COLOR:INTEGER;XL,YL,XU,YU:EXTENDED);
(* Writes in script the rectangle that must be plotted. (XL,YL) is the
   lowerleft corner and (XU,YU) the upper right corner of the rectangle
 *)
VAR S,SXL,SYL,SXU,SYU:ANSISTRING;
BEGIN
  WRITE(F_GRF,'set object ',OBJECT_NR,' rectangle from ');
  (* Convert XL and XU to wavenumbers. From X=h*nu/kT that would be
     wave=X*T/HCK (T>1) or wave=X/HCK (T<=1) in cm-1. UNIT_CALC_X=1/HCK
   *)
  IF TEMPERATURE>1 THEN XL:=XL*TEMPERATURE*UNIT_CAL_X ELSE XL:=XL*UNIT_CAL_X;
  IF TEMPERATURE>1 THEN XU:=XU*TEMPERATURE*UNIT_CAL_X ELSE XU:=XU*UNIT_CAL_X;
  STRING_NUMBER(XL,SXL); STRING_NUMBER(YL,SYL);
  STRING_NUMBER(XU,SXU); STRING_NUMBER(YU,SYU);
  WRITE(F_GRF,SXL,',',SYL);
  WRITE(F_GRF,' to ');
  WRITE(F_GRF,SXU,',',SYU);
  WRITE(F_GRF,' fc rgb');
  (* the color *)
  CASE COLOR OF
    0:S:='black';
    1:S:='dark-blue';
    2:S:='dark-green';
    3:S:='dark-cyan';
    4:S:='dark-red';
    5:S:='dark-magenta';
    6:S:='dark-orange';
    7:S:='grey';
    8:S:='dark-grey';
    9:S:='blue';
    10:S:='green';
    11:S:='cyan';
    12:S:='red';
    13:S:='magenta';
    14:S:='dark-yellow';
    15:S:='orange'
  END;
  WRITELN(F_GRF,' "',S,'" fillstyle solid 1.0')
END;

PROCEDURE SORT_FREQ(NUM:INTEGER);
(* Sort the frequencies in X_IN[1..NUM] in climbing order *)
VAR F_MIN,F_HELP:EXTENDED;
    J,I_MIN,J_MIN:INTEGER;
    LOWEST_FOUND:BOOLEAN;
BEGIN
  I_MIN:=1;
  REPEAT
    F_MIN:=X_IN[I_MIN]; LOWEST_FOUND:=FALSE;
    FOR J:=I_MIN+1 TO NUM DO
    BEGIN
      (* Determine lowest frequency *)
      IF X_IN[J]<F_MIN THEN
      BEGIN
        F_MIN:=X_IN[J]; J_MIN:=J; LOWEST_FOUND:=TRUE
      END
    END;
    (* Store the lowest frequency: interchange the continua and fractions *)
    IF LOWEST_FOUND=TRUE THEN
    BEGIN
      F_HELP:=X_IN[I_MIN]; X_IN[I_MIN]:=X_IN[J_MIN];
      X_IN[J_MIN]:=F_HELP
    END;
    (* In the nex cycle we must not consider the lowest width OC in I_MIN *)
    I_MIN:=I_MIN+1
  UNTIL
    I_MIN>NUM-1
END;

PROCEDURE GET_FREQ_BETWEEN(I:INTEGER;X_LEFT,X_RIGHT:EXTENDED;VAR NUM_IN:INTEGER);
(* Gets the frequencies between X_LEFT, X_RIGHT of OC number I. These
   frequencies are stored in X_IN[1..NUM]. We must consider the frequencies
   of all foregoing modes 1..I-1. We replaced the '<=' and '>=' signs in XiPT
   by '<' and '>' to get a smaller amount of boxes
 *)
VAR J:INTEGER;
BEGIN
  NUM_IN:=0;
  FOR J:=1 TO I-1 DO
  BEGIN
    (* Relation between OC number and frequency: Left cut-off frequency of mode
       OC is in XN[-1+2*OC]
     *)
    IF (XN[-1+2*J]>X_LEFT) AND (XN[-1+2*J]<X_RIGHT) THEN
    BEGIN
      NUM_IN:=NUM_IN+1; X_IN[NUM_IN]:=XN[-1+2*J]
    END;
    IF (XN[2*J]>X_LEFT) AND (XN[2*J]<X_RIGHT) THEN
    BEGIN
      NUM_IN:=NUM_IN+1; X_IN[NUM_IN]:=XN[2*J]
    END
  END;
  IF NUM_IN>1 THEN SORT_FREQ(NUM_IN);
  (* Put X_LEFT and X_RIGHT also in X_IN: they are the first and last
     element of the array
   *)
  IF NUM_IN>0 THEN
  BEGIN
    FOR J:=NUM_IN DOWNTO 1 DO X_IN[J+1]:=X_IN[J];
    NUM_IN:=NUM_IN+1;
    X_IN[1]:=X_LEFT;
    NUM_IN:=NUM_IN+1;
    X_IN[NUM_IN]:=X_RIGHT
  END
  ELSE
  BEGIN
    NUM_IN:=2; X_IN[1]:=X_LEFT; X_IN[2]:=X_RIGHT
  END
END;

PROCEDURE MAKE_SHORT_NAME(VAR NAME:ANSISTRING);
(* Input string NAME is stripped from its extension and string 'dat'
   is added: mgo.grf becomes mgo. The name 'mgo' is used for naming
   the datafiles in the script: mgo1.dat, mgo2.dat etc.
 *)
VAR S:ANSISTRING;
    J,LEN:INTEGER;
BEGIN
  LEN:=LENGTH(NAME);
  J:=1; S:='';
  REPEAT
    IF NAME[J]<>'.' THEN S:=S+NAME[J];
    J:=J+1
  UNTIL
    (NAME[J]='.') OR (J>LEN);
  NAME:=S
END;

PROCEDURE BUILD_VDOS;
(* Build the VDoS with coloured rectangles *)
VAR NAME_GRF,NAME_DAT,NAME,FILE_DIRECTORY:ANSISTRING;
    STRI:STRING[5];
    COLOR,OBJECT_NR,I,J,OC_NR,NUM,NUM_DAT:INTEGER;
    DELTA,XL,YL,XU,YU,XR,YR:EXTENDED;
    SYMBOL:ANSISTRING;
BEGIN
  NAME:=EXTRACTFILENAME(NAME_GNUPLOTFILE);
  (* Take the name of the script file without extension and put that in NAME *)
  MAKE_SHORT_NAME(NAME);
  (* File directory where the script file for GnuPlot is present *)
  FILE_DIRECTORY:=EXTRACTFILEDIR(NAME_GNUPLOTFILE);
  DELTA:=1E-6; SYMBOL:='Circle';
  IF NAME<>'0' THEN
  BEGIN
    NAME_GRF:=NAME_GNUPLOTFILE;
    (* Set Gnuplot script file open *)
    SETOUTGRF(NAME_GRF); HEAD_SCRIPT;
    (* Make the first OC which has the largest width *)
    OBJECT_NR:=1; COLOR:=1; OC_NR:=1;
    XL:=XN[-1+2*OC_NR]; YL:=0;
    XU:=XN[2*OC_NR]; YU:=OC_DOS(OC_NR,XU-DELTA);
    MAKE_OBJ_BOX(OBJECT_NR,COLOR,XL,YL,XU,YU);
    (* Now add all other OC's from 2 to N_EINSTEIN *)
    IF N_EINSTEIN>1 THEN
    BEGIN
      FOR I:=2 TO N_EINSTEIN DO
      BEGIN
        COLOR:=COLOR+1;
        IF COLOR>15 THEN COLOR:=1;
        (* Cut-off's for OC number I *)
        XR:=XN[-1+2*I]; YR:=XN[2*I];
        (* Determine all frequencies of the forgeoing OC's 1..i-1 having
           frequencies between XR and YR and sort them in climbing
           frequencies. NUM is the number of frequencies between XR and YR
           including XR and YR. The minimum number NUM=2. The number of new
           boxes is NUM-1. The frequencies including XR and YR are in
           X_IN[1..NUM]
         *)
        NUM:=2; GET_FREQ_BETWEEN(I,XR,YR,NUM);
        FOR J:=1 TO NUM-1 DO
        BEGIN
          (* Draw the new boxes due to continuum OC number I *)
          XL:=X_IN[J]; YL:=OC_DOS(I-1,XL+DELTA);
          XU:=X_IN[J+1]; YU:=OC_DOS(I,XU-DELTA);
          OBJECT_NR:=OBJECT_NR+1;
          MAKE_OBJ_BOX(OBJECT_NR,COLOR,XL,YL,XU,YU)
        END
      END
    END;
    NUM_DAT:=0;
    (* Now I must plot a point otherwise no box in the plot. *)
    NUM_DAT:=NUM_DAT+1; STR(NUM_DAT,STRI);
    NAME_DAT:=FILE_DIRECTORY+WHAT_SLASH+NAME+STRI+'.dat';
    SETOUTL(NAME_DAT);
    WRITELN(F_DAT,'0.0000 0.0000');
    CLOSE(F_DAT);
    (* Add this *.dat file to the script in NAME_DAT *)
    ADD_FILE_SCRIPT(NAME_DAT,0,12,'0');
    IF VDOS_PRESENT=TRUE THEN
    BEGIN
      (* Experimental data are converted to correct units in CONVERT_EXP_VDOS *)
      STRI:=''; NUM_DAT:=NUM_DAT+1; STR(NUM_DAT,STRI);
      NAME_DAT:=FILE_DIRECTORY+WHAT_SLASH+NAME+STRI+'.dat';
      SETOUTL(NAME_DAT);
      FOR J:=1 TO NUM_VDOS_POINTS DO
      BEGIN
        WRITELN(F_DAT,X_EXP[J]:12:5,' ',Y_EXP[J]:12:5);
      END;
      CLOSE(F_DAT);
      COLOR:=9;
      IF NUM_VDOS_POINTS>1 THEN ADD_FILE_SCRIPT(NAME_DAT,1,COLOR,SYMBOL)
      ELSE ADD_FILE_SCRIPT(NAME_DAT,0,COLOR,SYMBOL)
    END;
    WRITELN(F_GRF,'unset multiplot');
    FINALIZE_SCRIPT(NAME);
    (* Close the Gnuplot script .grf file *)
    CLOSE(F_GRF)
  END
END;

PROCEDURE MAKE_THETA(ETA:EXTENDED;MODE:INTEGER;VAR THETA_IN,THETA:EXTENDED);
(* Calculates the Einstein temperature THETA at compression ETA for the
   lower cut-off of the mode-th Einstein continuum. Section 6.1.
   This procedure differs from make_grun_theta in unit VIBRATIONS.PAS
   because we enter with a specific frequency THETA_IN that changes with ETA
 *)
VAR EX,Z1,Z,ZLN,POW_ETA_Q0,STRAIN,A1,A2,LN_ARG:EXTENDED;
BEGIN
  ZLN:=LN(ETA);
  IF GRUNEISEN_TYPE=1 THEN
  BEGIN
    (* Calculation of frequency: Al'tshuler expression, eqn. (6.7)+(6.8) *)
    POW_ETA_Q0:=EXP(Q0[MODE]*ZLN);
    IF ABS(Q0[MODE])<1E-8 THEN Z1:=-GAMMA0[MODE]*ZLN
    ELSE
    BEGIN
      Z1:=-GAMMA_INF[MODE]*ZLN-(GAMMA0[MODE]-GAMMA_INF[MODE])*(POW_ETA_Q0-1)/Q0[MODE]
    END
  END;
  IF GRUNEISEN_TYPE=2 THEN
  BEGIN
    (* Stixrude-Lithgow-Bertelloni (2005) elasticity formalism, eqn. (6.13)+(6.14) *)
    EX:=-2/3; Z:=POW_N(ETA,EX); STRAIN:=(Z-1)/2;
    A1:=6*GAMMA0[MODE];
    A2:=-12*GAMMA0[MODE]+36*GAMMA0[MODE]*GAMMA0[MODE]-18*GAMMA0[MODE]*Q0[MODE];
    (* Note that the argument of the LN can be negative when strain is negative.
       In that case we have imaginary frequency in eqn. (3.7) of Gruneis.doc. It
       occurs when volume is very large, far beyond melting and happens normally
       when V->infinity calculations.
       We set in that case that the argument LN_ARG is 1e-08 to prevent a
       runtime error.
       Eqn. (6.9)
     *)
    LN_ARG:=1+A1*STRAIN+0.5*A2*STRAIN*STRAIN;
    IF LN_ARG<1E-8 THEN LN_ARG:=1E-08;
    Z1:=0.5*LN(LN_ARG)
  END;
  (* Final result of the frequency. Argument, Z1, of exp must be larger than
     -max_arg_exp=-11200
   *)
  IF Z1<-MAX_ARG_EXP THEN Z1:=-MAX_ARG_EXP;
  THETA:=THETA_IN*EXP(Z1)
END;

PROCEDURE INIT_X_GAMMA(VPT:EXTENDED);
(* Initializes X=h*nu/k=theta for Einstein continua
   Einstein monochromators:   1       2       3 ......
   Einstein continua      :  X1 X2   X3 X4   X4 X5 ....
 *)
VAR ETA,X_DIV,XV:EXTENDED;
    IK,J:INTEGER;
BEGIN
  ETA:=VPT/V0; IK:=0;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    (* At volume VPT store Einstein temperatures of the continua in X_DUMMY. *)
    IK:=IK+1; MAKE_THETA(ETA,J,X0_DUMMY[IK],XV); X_DUMMY[IK]:=XV;
    (* Gamma,q are the same for begin and endpoint of continuum. Therefore
       the ratio X/X0 is the same for the lower cut-off and upper cut-off.
       For the upper cut-off only X0 is different.
     *)
    X_DIV:=X_DUMMY[IK]/X0_DUMMY[IK];
    IK:=IK+1;
    X_DUMMY[IK]:=X0_DUMMY[IK]*X_DIV
  END
END;

PROCEDURE PLOT_VDOS;
(* Plots the VDoS by making a Gnuplot file with coloured rectangles for
   the optic continua.
 *)
VAR OLD_PRESSURE,OLD_TEMPERATURE:EXTENDED;
    J:INTEGER;
BEGIN
  (* Check if user changed model parameters. In that case the static volume must be
     changed to keep a stable lattice at 0 K, 0 Pa.
     Initialize volume that must be kept in memory for making extrapolations
     and set extrapolation True only when more than 2 calculation have been
     carried out. Extrapolation speeds-up calculations.
     For static volume correction no extrapolation required.
   *)
  FIX_YES:=FALSE; EXTRAPOLATE:=FALSE; VMEMORY:=V0; OLD_VMEMORY:=V0; VPT:=V0;
  (* FIX_STATIC_VOLUME is in unit MAKEVOLUME.PAS. Note that FIX_STATIC_VOLUME
     requires PRESSURE=TEMPERATURE=0. 
   *)
  V0_ST_FIX:=V0_ST; OLD_PRESSURE:=PRESSURE; OLD_TEMPERATURE:=TEMPERATURE;
  V0_ST_ORIGINAL:=V0_ST;
  FIX_STATIC_VOLUME(V0_ST_FIX,FIX_YES);
  IF FIX_YES=TRUE THEN V0_ST:=V0_ST_FIX;
  PRESSURE:=OLD_PRESSURE; TEMPERATURE:=OLD_TEMPERATURE;
  (* Calculate volume VPT at PRESSURE, TEMPERATURE. *)
  CALC_VPT(VPT);
  (* Prepare frequencies and Gruneisen parameters. The frequencies are in
     X_DUMMY^[1..3+2*Noc+Neinstein] as h*nu/k
   *)
  INIT_X_GAMMA(VPT);
  (* Store the frequencies in a new array because we sort on width of the
     optical continua
   *)
  FOR J:=1 TO 2*N_EINSTEIN DO
  BEGIN
    IF TEMPERATURE>1 THEN XN[J]:=X_DUMMY[J]/TEMPERATURE ELSE XN[J]:=X_DUMMY[J]
  END;
  (* Store the fractions in a new array *)
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    FRAC_N[J]:=FRAC[J]
  END;
  (* Sort optic continua on width; the largest width OC is plotted first to
     give the nicest spectrum
   *)
  SORT_OC_WIDTH;
  (* Get the maximum frequency in desired unit and maximum value of the VDoS.
     Needed for HEAD_SCRIPT
   *)
  GET_MAX_FREQ;
  (* If experimental data are toggled on correct extrema *)
  IF VDOS_PRESENT=TRUE THEN MAXMIN;
  (* Build the VDoS *)
  BUILD_VDOS
END;

PROCEDURE UNIT_CONVERSION;
(* Convert to cm-1 or THz or meV *)
VAR HCK:EXTENDED;
BEGIN 
  (* Calculate the conversion from the calculated VDoS from X=h*nu/kT to
     the selected unit. VDOS_UNIT_DESIRED is used for easier distinguishing what
     unit must be used in GET_MAX_FREQ, OC_DOS, MAKE_OBJ_BOX
   *)
  CASE VDOS_UNIT_DESIRED OF
    1:BEGIN
        HCK:=100*PLANCK*LIGHT_SPEED/BOLTZMANN;
        UNIT_CAL_X:=1/HCK; UNIT_CAL_Y:=HCK
      END;
    2:BEGIN
        UNIT_CAL_X:=1E-12*BOLTZMANN/PLANCK; UNIT_CAL_Y:=1E+12*PLANCK/BOLTZMANN
      END;
    3:BEGIN
        UNIT_CAL_X:=1000*BOLTZMANN/ELEMENTARY_CHARGE;
        UNIT_CAL_Y:=ELEMENTARY_CHARGE/1000/BOLTZMANN
      END
  END;
  IF VDOS_PRESENT=TRUE THEN
  BEGIN
    (* Convert experimental data. cm-1, THz, meV compares to
       VDOS_UNIT_EXP=1,2,3
     *)
    CASE VDOS_UNIT_EXP OF
      1:BEGIN
          CASE VDOS_UNIT_DESIRED OF
            1:UNIT_EXP_X:=1;
            2:UNIT_EXP_X:=100*LIGHT_SPEED*1E-12;
            3:UNIT_EXP_X:=PLANCK*LIGHT_SPEED*1E+05/ELEMENTARY_CHARGE
          END
        END;
      2:BEGIN
          CASE VDOS_UNIT_DESIRED OF
            1:UNIT_EXP_X:=1E+10/LIGHT_SPEED;
            2:UNIT_EXP_X:=1;
            3:UNIT_EXP_X:=1E+15*PLANCK/ELEMENTARY_CHARGE
          END
        END;
      3:BEGIN
          CASE VDOS_UNIT_DESIRED OF
            1:UNIT_EXP_X:=1E-05/ELEMENTARY_CHARGE/PLANCK/LIGHT_SPEED;
            2:UNIT_EXP_X:=1E-15/ELEMENTARY_CHARGE/PLANCK;
            3:UNIT_EXP_X:=1
          END
        END
    END;
    (* To have dimensionless surface under the VDOS make UNIT_EXP_Y=1/UNIT_EXP_X *)
    UNIT_EXP_Y:=1/UNIT_EXP_X
  END
END;

PROCEDURE SORT_X_VALUES(VAR X,Y:VDOSPOINTS;NUM:INTEGER);
(* Sort (X,Y) VDoS points in climbing X. (X,Y) values are stored in arrays with
   NUM elements (maximum MXADD=2000)
 *)
VAR X_MINIMUM,X_HELP:EXTENDED;
    J,I_MIN,J_MIN:INTEGER;
    LOWEST_FOUND:BOOLEAN;
BEGIN
  I_MIN:=1;
  REPEAT
    X_MINIMUM:=X[I_MIN]; LOWEST_FOUND:=FALSE;
    FOR J:=I_MIN+1 TO NUM DO
    BEGIN
      (* Determine the lowest X-value *)
      IF X[J]<X_MINIMUM THEN
      BEGIN
        X_MINIMUM:=X[J]; J_MIN:=J; LOWEST_FOUND:=TRUE
      END
    END;
    (* Store lowest X-value: interchange the X,Y values with that in I_MIN *)
    IF LOWEST_FOUND=TRUE THEN
    BEGIN
      X_HELP:=X[I_MIN]; X[I_MIN]:=X[J_MIN]; X[J_MIN]:=X_HELP;
      X_HELP:=Y[I_MIN]; Y[I_MIN]:=Y[J_MIN]; Y[J_MIN]:=X_HELP
    END;
    (* In the next cycle do not consider the lowest X in I_MIN *)
    I_MIN:=I_MIN+1
  UNTIL
    I_MIN>NUM-1
END;

FUNCTION CALC_SURFACE(X,Y:VDOSPOINTS;NUM:INTEGER):REAL;
(* Calculation of the surface underneath (X,Y) points. These points are sorted in
   climbing X and connected with lines. The surface under that plot
   is calculated. NUM is the number of (X,Y) points
 *)
VAR J:INTEGER;
    SUM,SURF,RECT,TRIAN:EXTENDED;
BEGIN
  SUM:=0;
  FOR J:=2 TO NUM DO
  BEGIN
    IF Y[J]>Y[J-1] THEN
    BEGIN
      (* Climbing part of the curve. RECT is the surface of the rectangle and
         TRIAN is the surface of the remaining triangle
       *)
      RECT:=(X[J]-X[J-1])*Y[J-1]; 
      TRIAN:=0.5*(X[J]-X[J-1])*ABS(Y[J]-Y[J-1])
    END
    ELSE
    BEGIN
      (* Descending part of the curve *)
      RECT:=(X[J]-X[J-1])*Y[J];
      TRIAN:=0.5*(X[J]-X[J-1])*ABS(Y[J-1]-Y[J])
    END;
    (* Surf is surface of rectangle plus triangle *)
    SURF:=RECT+TRIAN; SUM:=SUM+SURF
  END;
  CALC_SURFACE:=SUM
END;

PROCEDURE CONVERT_EXP_VDOS;
(* Converts units experimental/abinito VDoS and normalizes the surface under
   the VDoS to 3n. The Y-value is g/N_A where N_A is Avogadro's number. The
   result is stored in X_EXP,Y_EXP
 *)
VAR FACTOR,SURFACE:EXTENDED;
    J:INTEGER;
BEGIN
  (* Sort the X,Y VDoS values in climbing X *)
  SORT_X_VALUES(X_EXP,Y_EXP,NUM_VDOS_POINTS);
  FOR J:=1 TO NUM_VDOS_POINTS DO
  BEGIN
    X_EXP[J]:=X_EXP[J]*UNIT_EXP_X; Y_EXP[J]:=Y_EXP[J]*UNIT_EXP_Y
  END;
  (* Calculate surface under experimental VDoS *)
  SURFACE:=CALC_SURFACE(X_EXP,Y_EXP,NUM_VDOS_POINTS);
  (* Normalize the Y-values such that the surface is 3nN_a *)
  FACTOR:=3*N_MOL/SURFACE;
  FOR J:=1 TO NUM_VDOS_POINTS DO Y_EXP[J]:=Y_EXP[J]*FACTOR
END;

PROCEDURE MAKE_CONTINUA;
(* The Einstein temperatures in this program were derived from Kieffer's continuum
   model in which each continuum has a lower and upper cut-off frequency. Now we
   must follow the reverse way to get the continua back composed of 2*N_EINSTEIN
   frequencies
 *)
VAR DELTA,DELTA1,DELTA2,CRIT:EXTENDED;
    I,IK:INTEGER;
BEGIN
  (* Write the optic in Einstein temperatures *)
  IK:=0; CRIT:=1E-05;
  FOR I:=1 TO N_EINSTEIN DO
  BEGIN
    (* Spacing between subsequent Einstein frequencies, DELTA1 and DELTA2 *)
    DELTA1:=1E+20; DELTA2:=1E+20;
    IF I>1 THEN DELTA1:=THETA0[I]-THETA0[I-1];
    IF I<N_EINSTEIN THEN DELTA2:=THETA0[I+1]-THETA0[I];
    IF DELTA1<DELTA2 THEN DELTA:=DELTA1 ELSE DELTA:=DELTA2;
    DELTA:=DELTA/2;
    IK:=IK+1; X0_DUMMY[IK]:=THETA0[I]-DELTA;
    IF X0_DUMMY[IK]<CRIT THEN X0_DUMMY[IK]:=CRIT;
    IK:=IK+1; X0_DUMMY[IK]:=THETA0[I]+DELTA;
  END
END;

PROCEDURE MAKE_VDOS;
(* Making a VDoS of Kieffer's model for a stoichiometric compound *)
VAR SHORT_NAME:ANSISTRING;
    J:INTEGER;
BEGIN
  (* Initialize maximum and minimum values of frequencies and VDoS *)
  X_MIN:=0; Y_MIN:=0; X_MAX:=0; Y_MAX:=0;
  (* Determine unit conversion factors *)
  UNIT_CONVERSION;
  (* If present convert units experimental/abinitio VDoS *)
  IF VDOS_PRESENT=TRUE THEN CONVERT_EXP_VDOS;
  (* From monochromatic Einstein to Einstein continua *)
  MAKE_CONTINUA;
  (* Plot VDoS *)
  PLOT_VDOS;
  (* GnuPlot files now ready *)
  FOR J:=1 TO 51 DO WRITE('-');
  WRITELN;
  WRITELN('Input file: ',NAME_INPUTFILE);
  WRITELN('Created   : ',NAME_GNUPLOTFILE);
  SHORT_NAME:=EXTRACTFILENAME(NAME_GNUPLOTFILE);
  WRITELN('Use gnuplot by typing: gnuplot ',SHORT_NAME,' -');
  WRITELN('(don''t forget the trailing dash in that command...)');
  WRITELN('After plotting press ''q'' to quit GnuPlot...');
  WRITELN('Normal termination...');
  FOR J:=1 TO 51 DO WRITE('-');
  WRITELN; WRITELN
END;

END.
