UNIT static;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   calculating properties of the static lattice in the multiple Einstein method
 *)

INTERFACE
USES CRT,sysutils,variables;


FUNCTION STATIC_HELMHOLTZ(VPT:EXTENDED):EXTENDED;
FUNCTION STATIC_BULKMOD(VPT:EXTENDED):EXTENDED;
FUNCTION STATIC_SHEARMODULUS(VPT:EXTENDED):EXTENDED;
IMPLEMENTATION


FUNCTION STATIC_HELMHOLTZ(VPT:EXTENDED):EXTENDED;
(* Static Helmholtz energy without reference energy, U_REF.
   EOS_TYPE=1: 3rd order Vinet
   EOS_TYPE=2: N-th order Birch-Murnaghan up to MAX_EOS_ORDER, gives
   MAX_EOS_ORDER-1 coefficients in the strain equation because BMN[2]=1.
   EOS_ORDER is the exponent in strain in the Helmholtz equation.
   EOS_TYPE=3: Keane's (1954) EoS
   EOS_TYPE=4: Qin's (2008) EoS
   At V=V0st the Helmholtz energy is zero
 *)
TYPE RY_BIRCH=ARRAY[2..MAX_EOS_ORDER] OF EXTENDED;
VAR BMN:RY_BIRCH;
    A,EX,SUM,Z,ZY,A1,STRAIN:EXTENDED;
    J:INTEGER;
BEGIN
  (* Functions POW_N, INT_POW_N are in unit VARIABLES.PAS *)
  CASE EOS_TYPE OF
    1:BEGIN
        (* 3rd order Vinet's EOS, FreePascal Manual, Eq. (5.12, 5.13) *)
        IF ABS(DK0_ST[1]-1)>1E-16 THEN
        BEGIN
          A:=1.5*(DK0_ST[1]-1); Z:=VPT/V0_ST;
          EX:=1/3; Z:=1-POW_N(Z,EX); Z:=Z*A;
          A1:=1+(Z-1)*EXP(Z); A:=DK0_ST[1]-1;
          STATIC_HELMHOLTZ:=4*K0_ST*V0_ST*A1/A/A
        END
        ELSE
        BEGIN
          Z:=VPT/V0_ST; EX:=1/3; Z:=POW_N(Z,EX);
          STATIC_HELMHOLTZ:=-9*K0_ST*V0_ST*(Z-Z*Z/2-0.5)
        END
      END;
    2:BEGIN
        IF EOS_ORDER=3 THEN
        BEGIN
          (* 3rd order Birch-Murnaghan *)
          A:=1.5*(DK0_ST[1]-4); Z:=VPT/V0_ST;
          EX:=-2/3; ZY:=POW_N(Z,EX)-1; ZY:=ZY/2;
          SUM:=9*K0_ST*V0_ST*(ZY*ZY/2+A*ZY*ZY*ZY/3);
          STATIC_HELMHOLTZ:=SUM
        END
        ELSE
        BEGIN
          (* N-th order BM EoS with strain coefficients 2 to EOS_ORDER.
             BMN[2]=1. There are 1..EOS_ORDER-2 coefficients dealing with
             derivatives of bulk modulus, DK0_ST[J]. Strain is 'STRAIN'.
             FreePascal Manual, Eq. (5.7)
           *)
          FOR J:=1 TO EOS_ORDER-2 DO BMN[J+2]:=DK0_ST[J];
          BMN[2]:=1;
          Z:=VPT/V0_ST; EX:=-2/3; STRAIN:=(POW_N(Z,EX)-1)/2;
          SUM:=0;
          FOR J:=2 TO EOS_ORDER DO SUM:=SUM+BMN[J]*INT_POW_N(STRAIN,J)/J;
          STATIC_HELMHOLTZ:=9*K0_ST*V0_ST*SUM
        END
      END;
    3:BEGIN
        (* Keane's EOS: taken as 4th order, FreePascal Manual Eq. (5.18a,b) *)
        Z:=VPT/V0_ST;
        IF ABS(DK0_ST[2]-1)>1E-16 THEN
        BEGIN
          A:=1-DK0_ST[2]; A:=1-POW_N(Z,A); 
          A:=V0_ST*A/(1-DK0_ST[2])+VPT-V0_ST;
          A:=DK0_ST[1]*A/DK0_ST[2]/DK0_ST[2];
          A1:=VPT*LN(Z)-VPT+V0_ST; A1:=A1*(DK0_ST[1]/DK0_ST[2]-1); A:=A-A1;
          STATIC_HELMHOLTZ:=K0_ST*A
        END
        ELSE
        BEGIN
          A:=DK0_ST[1]*(LN(Z)-Z+1)+(DK0_ST[1]-1)*(Z*LN(Z)-Z+1);
          STATIC_HELMHOLTZ:=-K0_ST*V0_ST*A
        END
      END;
    4:BEGIN
        (* Qin et al 's (2008) EoS, FreePascal Manual Eq. (5.22) *)
        Z:=VPT/V0_ST; EX:=1/3; STRAIN:=POW_N(Z,EX);
        A:=EXP(-DK0_ST[1]*(STRAIN-1))/POW_N(STRAIN,DK0_ST[3])-1;
        A1:=EXP(-DK0_ST[2]*(STRAIN-1))*POW_N(STRAIN,DK0_ST[4])-1;
        A1:=-A1*(DK0_ST[1]+DK0_ST[3])/(DK0_ST[2]-DK0_ST[4]);
        SUM:=A+A1;
        A:=(DK0_ST[1]+DK0_ST[3])*(DK0_ST[2]-DK0_ST[4])*(DK0_ST[1]+DK0_ST[3]-DK0_ST[2]+DK0_ST[4]);
        A:=A+DK0_ST[1]*DK0_ST[4]+DK0_ST[2]*DK0_ST[3];
        A:=9*K0_ST*V0_ST*(DK0_ST[2]-DK0_ST[4])/A;
        STATIC_HELMHOLTZ:=A*SUM
      END
  END {OF CASE}
END;

FUNCTION STATIC_BULKMOD(VPT:EXTENDED):EXTENDED;
(* Static Bulk modulus.
   EOS_TYP=2: N-th order Birch-Murnaghan up to MAX_EOS_ORDER, gives
   MAX_EOS_ORDER-1 coefficients in the strain equation because BMN[0]=1.
   EOS_ORDER is the exponent in strain in the Helmholtz equation.
   EOS_TYP=1: 3rd order Vinet.
   At V=V0st the Bulk modulus is K0_ST
 *)
TYPE RY_BIRCH=ARRAY[2..MAX_EOS_ORDER] OF EXTENDED;
VAR BMN:RY_BIRCH;
    A,EX,SUM1,SUM2,Z,Z1,Z2,Z5,STRAIN:EXTENDED;
    J:INTEGER;
BEGIN
  (* Functions POW_N, INT_POW_N are in unit VARIABLES.PAS *)
  CASE EOS_TYPE OF
    1:BEGIN
        (* 3rd order Vinet EOS, FreePascal Manual Eq. (5.15) *)
        A:=DK0_ST[1]-1; Z:=VPT/V0_ST; EX:=1/3; Z:=POW_N(Z,EX);
        Z1:=1/Z; Z2:=Z1*Z1;
        SUM1:=2*Z2-Z1+1.5*(Z1-1)*A; SUM2:=1.5*A*(1-Z);
        IF SUM2>MAX_ARG_EXP THEN SUM2:=MAX_ARG_EXP
        ELSE
          IF SUM2<-MAX_ARG_EXP THEN SUM2:=-MAX_ARG_EXP;
        STATIC_BULKMOD:=K0_ST*SUM1*EXP(SUM2)
      END;
    2:BEGIN
        IF EOS_ORDER=3 THEN
        BEGIN
          (* 3rd order Birch-Murnaghan EOS: coefficients bmn[3] is the pressure
             derivative of bulk modulus
           *)
          BMN[2]:=1; BMN[3]:=1.5*(DK0_ST[1]-4);
          Z:=VPT/V0_ST; EX:=-2/3; STRAIN:=(POW_N(Z,EX)-1)/2;
          Z2:=1+2*STRAIN; EX:=5/2; Z5:=POW_N(Z2,EX);
          SUM1:=0; SUM2:=0;
          FOR J:=2 TO 3 DO SUM1:=SUM1+BMN[J]*INT_POW_N(STRAIN,J-1);
          FOR J:=2 TO 3 DO SUM2:=SUM2+BMN[J]*(J-1)*INT_POW_N(STRAIN,J-2);
          STATIC_BULKMOD:=K0_ST*Z5*(5*SUM1+Z2*SUM2)
        END
        ELSE
        BEGIN
          (* N-th order BM EoS with strain coefficients 2 to EOS_ORDER.
             BMN[2]=1. There are 1..EOS_ORDER-2 coefficients dealing with
             derivatives of bulk modulus, DK0_ST[J]. Strain is 'STRAIN'.
             FreePascal Manual, Eq. (5.9)
           *)
          FOR J:=1 TO EOS_ORDER-2 DO BMN[J+2]:=DK0_ST[J];
          BMN[2]:=1;
          Z:=VPT/V0_ST; EX:=-2/3; STRAIN:=(POW_N(Z,EX)-1)/2;
          Z2:=1+2*STRAIN; EX:=5/2; Z5:=POW_N(Z2,EX);
          SUM1:=0; SUM2:=0;
          FOR J:=2 TO EOS_ORDER DO SUM1:=SUM1+BMN[J]*INT_POW_N(STRAIN,J-1);
          FOR J:=2 TO EOS_ORDER DO SUM2:=SUM2+BMN[J]*(J-1)*INT_POW_N(STRAIN,J-2);
          STATIC_BULKMOD:=K0_ST*Z5*(5*SUM1+Z2*SUM2)
        END
      END;
    3:BEGIN
        (* Keane's (1954) EoS, FreePascal Manual Eq. (5.20) *)
        Z:=VPT/V0_ST; Z:=POW_N(Z,-DK0_ST[2])-1; Z:=DK0_ST[1]*Z/DK0_ST[2]+1;
        STATIC_BULKMOD:=K0_ST*Z
      END;
    4:BEGIN
        (* Qin et al's (2008) EoS, FreePascal Manual Eq. (5.25) *)
        Z:=VPT/V0_ST; EX:=1/3; STRAIN:=POW_N(Z,EX);
        EX:=-DK0_ST[3]-1;
        Z2:=(DK0_ST[2]-DK0_ST[4])*POW_N(STRAIN,EX)*EXP(-DK0_ST[1]*(STRAIN-1));
        Z2:=Z2*(DK0_ST[3]*(DK0_ST[3]+3)/STRAIN/STRAIN+2*DK0_ST[1]*(DK0_ST[3]+1)/STRAIN+DK0_ST[1]*DK0_ST[1]);
        EX:=DK0_ST[4]-1;
        Z5:=(DK0_ST[1]+DK0_ST[3])*POW_N(STRAIN,EX)*EXP(-DK0_ST[2]*(STRAIN-1));
        Z5:=Z5*(-DK0_ST[4]*(DK0_ST[4]-3)/STRAIN/STRAIN+2*DK0_ST[2]*(DK0_ST[4]-1)/STRAIN-DK0_ST[2]*DK0_ST[2]);
        Z2:=Z2+Z5;
        Z5:=(DK0_ST[1]+DK0_ST[3])*(DK0_ST[2]-DK0_ST[4])*(DK0_ST[1]+DK0_ST[3]-DK0_ST[2]+DK0_ST[4]);
        Z5:=Z5+DK0_ST[1]*DK0_ST[4]+DK0_ST[2]*DK0_ST[3];
        Z5:=K0_ST/Z5;
        STATIC_BULKMOD:=Z5*Z2
      END
  END {OF CASE}
END;

FUNCTION FACULTY(N:INTEGER):INTEGER;
(* Calculates faculty(N)=Nx(N-1)x(N-2)x...x2x1 *)
VAR FAC:INTEGER;
BEGIN
  IF (N=0) OR (N=1) THEN FACULTY:=1
  ELSE
  BEGIN
    FAC:=N;
    REPEAT
      N:=N-1; FAC:=FAC*N
    UNTIL
      N<=1;
    FACULTY:=FAC
  END
END;

FUNCTION STATIC_SHEARMODULUS(VPT:EXTENDED):EXTENDED;
(* Static shear modulus: finite strain theory *)
TYPE RY_BIRCH=ARRAY[2..MAX_EOS_ORDER] OF EXTENDED;
VAR BMN,GMN:RY_BIRCH;
    PRE_FAC,EX,Z,Z2,STRAIN,ETA,MULT_G,MULT_K,SUM_G,SUM_K:EXTENDED;
    J:INTEGER;
BEGIN
  (* Functions POW_N, INT_POW_N are in unit VARIABLES.PAS *)
  IF EOS_TYPE=2 THEN
  BEGIN
    (* Equation of state based on finite strain *)
    IF EOS_ORDER=3 THEN
    BEGIN
      (* As given by Stixrude and Bertelloni (2005) GJI 162:610-632 *)
      ETA:=VPT/V0_ST; EX:=-2/3; Z:=POW_N(ETA,EX); STRAIN:=(Z-1)/2;
      Z2:=1+2*STRAIN; EX:=2.5; PRE_FAC:=POW_N(Z2,EX);
      Z:=G0_ST+(3*K0_ST*DG0_ST[1]-5*G0_ST)*STRAIN;
      Z:=Z+(6*K0_ST*DG0_ST[1]-24*K0_ST-14*G0_ST+4.5*K0_ST*DK0_ST[1])*STRAIN*STRAIN;
      STATIC_SHEARMODULUS:=PRE_FAC*Z
    END
    ELSE
    BEGIN
      (* N-th order BM EoS with strain coefficients 2 to EOS_ORDER.
         BMN[2]=1, GMN[2]=1. There are 1..EOS_ORDER-2 coefficients dealing with
         derivatives of bulk/shear modulus, DK0_ST[J], DG0_ST[J]. Strain is 'STRAIN'.
       *)
      FOR J:=1 TO EOS_ORDER-2 DO BMN[J+2]:=DK0_ST[J];
      BMN[2]:=1;
      (* N-th order pressure coefficients shear modulus, see me-freepascal-demo.pdf,
         GMN[2]=1
         order: 2  3  4  5  6  7  8
         gmn  : 2  3  4  5  6  7  8
       *)
      FOR J:=1 TO EOS_ORDER-2 DO GMN[J+2]:=DG0_ST[J];
      GMN[2]:=1;
      ETA:=VPT/V0_ST; EX:=-2/3; Z:=POW_N(ETA,EX); STRAIN:=(Z-1)/2;
      Z2:=1+2*STRAIN; EX:=3.5; MULT_G:=POW_N(Z2,EX);
      EX:=2.5; MULT_K:=POW_N(Z2,EX);
      (* Summation for shear modulus *)
      SUM_G:=0;
      FOR J:=2 TO EOS_ORDER DO SUM_G:=SUM_G+GMN[J]*INT_POW_N(STRAIN,J-2);
      (* Summation for part with K0st *)
      SUM_K:=0;
      FOR J:=2 TO EOS_ORDER DO SUM_K:=SUM_K+BMN[J]*INT_POW_N(STRAIN,J-1);
      STATIC_SHEARMODULUS:=MULT_G*G0_ST*SUM_G+3*K0_ST*MULT_K*SUM_K
    END
  END
  ELSE
  BEGIN
    (* Shear modulus has not been programmed for EoS that is not based
       on finite strain, therefore it is set to zero
     *)
    STATIC_SHEARMODULUS:=0
  END
END;

END.
