UNIT vibrations;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Calculating the vibrational contribution to thermodynamic properties
   in the multiple Einstein method.
   Equations refer to manual, chapter 6
 *)

INTERFACE
USES CRT,sysutils,variables;


FUNCTION THERMAL_PRESSURE(VPT:EXTENDED):EXTENDED;
PROCEDURE VIBRATION_PROPERTIES(VPT:EXTENDED;VAR ENERGY,ENTROPY,CV,DPDT,
                               HELMHOLTZ,BULKMOD,ZERO_PNT:EXTENDED);
PROCEDURE SHEAR_VIB(VPT:EXTENDED;VAR GSH_VIB:EXTENDED);
IMPLEMENTATION


FUNCTION ANH_A(MODE,WHAT:INTEGER;VPT:EXTENDED):EXTENDED;
(* Anharmonicity parameter from perturbation theory: a=a0(V/V0)^z for a mode.
   What=0 gives a. What=1 gives a'=z*a/V. What=2 gives a"=z*(z-1)*a/V^2.
   Volume is VPT. V0 is the volume at 0 Kelvin, 0 Pascal
 *)
VAR A,ETA,A0,Z0:EXTENDED;
BEGIN
  ETA:=VPT/V0;
  A0:=ANH_A0[MODE]; Z0:=ANH_Z0[MODE];
  (* Eqn. (6.18) *)
  A:=A0*POW_N(ETA,Z0);
  IF WHAT=0 THEN ANH_A:=A
  ELSE
  BEGIN
    IF WHAT=1 THEN
    BEGIN
      (* Eqn. (6.19) *)
      ANH_A:=Z0*A/VPT
    END
    ELSE
    BEGIN
      (* Eqn. (6.20) *)
      ANH_A:=Z0*(Z0-1)*A/VPT/VPT
    END
  END
END;

FUNCTION P_ANHARMONIC(MODE:INTEGER;GAMMA,VPT,X:EXTENDED):EXTENDED;
(* Anharmonic thermal pressure contribution resulting from the Mode-th Einstein
   frequency. Without factor 3nRT/V. 
   X=Einstein temperature/T = theta/T.
   Note that in THERMAL_PRESSURE we multiply with 3nRT/V. Therefore we multiply
   with only TEMPERATURE and not with the squared TEMPERATURE
 *)
VAR X2,Z,Z2,Z3,ZA,ZB,A,A1:EXTENDED;
BEGIN
  (* Anharmonicity and its 1st volume derivative *)
  A:=ANH_A(MODE,0,VPT); A1:=ANH_A(MODE,1,VPT);
  IF X>CRIT_ARG_EXP THEN
  BEGIN
    IF X<MAX_ARG_EXP THEN
    BEGIN
      (* Normal values for X, eqn. (6.25a) 2nd term. We must restrict the value
         for exp(x) because powers are taken and the limit for extended real is 1e+4950
       *)
      Z:=EXP(X);
      IF Z>RESTRICT_EXP THEN Z:=RESTRICT_EXP;
      Z2:=(Z-1)*(Z-1); Z3:=Z2*(Z-1); X2:=X*X;
      ZA:=X2/2+3*X2*((2-X)*Z*Z-(2+X)*Z)/Z3;
      ZB:=X2/4+3*X2*Z/Z2;
      P_ANHARMONIC:=(A*GAMMA*ZA-A1*VPT*ZB)*TEMPERATURE/6
    END
    ELSE
    BEGIN
      (* Extremely large values for X=h*nu/kT, see eqn. (6.25d) *)
      P_ANHARMONIC:=(A*GAMMA/12-VPT*A1/24)*TEMPERATURE*X*X
    END
  END
  ELSE
  BEGIN
    (* For small X the exp() function becomes unstable and the limiting x->0 value
       is taken. See eqn. (6.25e)
     *)
    P_ANHARMONIC:=-0.5*TEMPERATURE*A1*VPT
  END
END;

FUNCTION CV_ANHARMONIC(MODE:INTEGER;T_IN,VPT,X:EXTENDED):EXTENDED;
(* Anharmonic CV contribution resulting from the Mode-th Einstein frequency.
   X=Einstein temperature/T = theta/T. T_IN is incoming temperature
   Without factor 3nR.
 *)
VAR X3,Z,Z2,ZA,ZB,A:EXTENDED;
BEGIN
  (* Anharmonicity parameter *)
  A:=ANH_A(MODE,0,VPT);
  IF X>CRIT_ARG_EXP THEN
  BEGIN
    IF X<MAX_ARG_EXP THEN
    BEGIN
      (* Normal values for X, see eqn. (6.24a), 2nd term. We must restrict the value
         for exp(x) because powers are taken and the limit for extended real is 1e+4950
       *)
      Z:=EXP(X);
      IF Z>RESTRICT_EXP THEN Z:=RESTRICT_EXP;
      Z2:=Z*Z; X3:=X*X*X;
      ZA:=X3*Z*(Z2*(X-2)+4*X*Z+2+X);
      ZB:=Z-1; ZB:=POW_N(ZB,4);
      CV_ANHARMONIC:=-A*T_IN*ZA/ZB/2
    END
    ELSE
    BEGIN
      (* Extremely large values for X, eqn. (6.24d) *)
      CV_ANHARMONIC:=0
    END
  END
  ELSE
  BEGIN
    (* For small X the exp() function becomes unstable and the limiting x->0 value
       is taken, eqn. (6.24e)
     *)
    CV_ANHARMONIC:=-T_IN*A
  END
END;

FUNCTION U_ANHARMONIC(MODE:INTEGER;T_IN,VPT,X:EXTENDED):EXTENDED;
(* Anharmonic energy contribution resulting from the Mode-th Einstein frequency.
   X=Einstein temperature/T = theta/T. T_IN is incoming temperature
   Without factor 3nRT.
 *)
VAR X2,X3,Z,Z2,ZA,ZB,A:EXTENDED;
BEGIN
  A:=ANH_A(MODE,0,VPT);
  IF X>CRIT_ARG_EXP THEN
  BEGIN
    IF X<MAX_ARG_EXP THEN
    BEGIN
      (* Normal values for X, see eqn. (6.23) 2nd term. We must restrict the value
         for exp(x) because powers are taken and the limit for extended real is 1e+4950
       *)
      Z:=EXP(X);
      IF Z>RESTRICT_EXP THEN Z:=RESTRICT_EXP;
      X2:=X*X; X3:=X2*X;
      Z2:=Z-1; ZA:=Z2*Z2; ZB:=ZA*Z2;
      Z2:=X2/4+3*X2*Z/ZA-3*X3*Z*(Z+1)/ZB;
      U_ANHARMONIC:=T_IN*A*Z2/6
    END
    ELSE
    BEGIN
      (* Extremely large values for X, eqn. (6.23d) *)
      U_ANHARMONIC:=T_IN*A*X*X/24
    END
  END
  ELSE
  BEGIN
    (* For small X the exp() function becomes unstable and the limiting x->0 value
       is taken, eqn. (6.23e)
     *)
    U_ANHARMONIC:=-T_IN*A/2
  END
END;

FUNCTION DP_ANHARMONIC(MODE:INTEGER;T_IN,GAMMA,VPT,X:EXTENDED):EXTENDED;
(* Anharmonic (dP/dT)V contribution resulting from the Mode-th Einstein
   frequency. 
   X=Einstein temperature/T = theta/T. T_IN is incoming temperature
   Without factor 3nR/V. See anhar.doc.
 *)
VAR X3,Z,Z2,ZA,ZB,ZC,A,A1:EXTENDED;
BEGIN
  A:=ANH_A(MODE,0,VPT); A1:=ANH_A(MODE,1,VPT);
  IF X>CRIT_ARG_EXP THEN
  BEGIN
    IF X<MAX_ARG_EXP THEN
    BEGIN
      (* Normal, not extreme values of X, eqn. (6.26) 2nd term. We must
         restrict the value for exp(x) because powers are taken and the limit for
         extended real is 1e+4950
       *)
      Z:=EXP(X);
      IF Z>RESTRICT_EXP THEN Z:=RESTRICT_EXP;
      Z2:=Z*Z; X3:=X*X*X;
      ZA:=X3*Z*(Z2*(3-X)-4*X*Z-3-X);
      ZB:=X3*Z*(Z+1);
      ZC:=Z-1;
      DP_ANHARMONIC:=A*T_IN*GAMMA*ZA/2/POW_N(ZC,4)-A1*T_IN*VPT*ZB/2/POW_N(ZC,3)
    END
    ELSE
    BEGIN
      (* Extremenly large value of X, eqn. (6.26d) *)
      DP_ANHARMONIC:=0
    END
  END
  ELSE
  BEGIN
    (* For small X the exp() function becomes unstable and the limiting x->0 value
       is taken. See eqn. (6.26e)
     *)
    DP_ANHARMONIC:=-T_IN*A1*VPT
  END
END;

FUNCTION K_ANHARMONIC(MODE:INTEGER;T_IN,GAMMA,DGDV,VPT,X:EXTENDED):EXTENDED;
(* Anharmonic isothermal bulk modulus contribution resulting from the
   Mode-th Einstein frequency.
   X=Einstein temperature/T = theta/T. T_IN is incoming temperature
   Without factor 3nRT/V. See anhar.doc.
 *)
VAR Z,Z2,ZA,ZB,ZC,ZD3,ZD4,ZE,ZF,ZG,A,A1,A2,X2:EXTENDED;
BEGIN
  A:=ANH_A(MODE,0,VPT); A1:=ANH_A(MODE,1,VPT); A2:=ANH_A(MODE,2,VPT);
  IF X>CRIT_ARG_EXP THEN
  BEGIN
    IF X<MAX_ARG_EXP THEN
    BEGIN
      (* Normal, not extreme values for X: eqn. (6.27a). Note that
         dgdv=-V*(dGamma/dV) at constant T. We must restrict the value for exp(x) because
         powers are taken and the limit for extended real is 1e+4950
       *)
      Z:=EXP(X);
      IF Z>RESTRICT_EXP THEN Z:=RESTRICT_EXP;
      Z2:=Z*Z; X2:=X*X; ZC:=Z-1;
      ZA:=3*X2*Z*(Z*(2-X)-(2+X)); ZD3:=POW_N(ZC,3); ZE:=ZA/ZD3+X2/2;
      ZA:=A*(GAMMA+DGDV)*ZE;
      ZB:=3*Z*X2*(Z2*(4-5*X+X2)+Z*(-8+4*X2)+4+5*X+X2); ZD4:=POW_N(ZC,4);
      ZB:=(ZB/ZD4+X2)*A*GAMMA*GAMMA;
      ZF:=X2/4+3*X2*Z/ZC/ZC; ZF:=ZF*A2*VPT*VPT;
      ZG:=GAMMA*A1*VPT*ZE;
      K_ANHARMONIC:=(ZA+ZB+ZF-ZG)*T_IN/6
    END
    ELSE
    BEGIN
      (* Extremely large values for X, eqn. (6.27d) *)
      ZA:=A*(GAMMA+DGDV)/12+A*GAMMA*GAMMA/6+A2*VPT*VPT/24-GAMMA*A1*VPT/12;
      K_ANHARMONIC:=T_IN*X*X*ZA
    END
  END
  ELSE
  BEGIN
    (* For small X the exp() function becomes unstable and the limiting x->0 value
       is taken. See eqn. (6.27e)
     *)
    K_ANHARMONIC:=0.5*T_IN*VPT*VPT*A2
  END
END;

PROCEDURE MAKE_GRUN_THETA(ETA:EXTENDED;MODE:INTEGER;VAR GAMMA,THETA:EXTENDED);
(* Calculates the Gruneisen parameter, GAMMA and Einstein temperature THETA
   at compression ETA for the mode-th Einstein frequency. Section 6.1
 *)
VAR EX,Z1,Z,ZLN,POW_ETA_Q0,STRAIN,A1,A2,LN_ARG:EXTENDED;
BEGIN
  ZLN:=LN(ETA);
  IF GRUNEISEN_TYPE=1 THEN
  BEGIN
    (* Calculation of frequency: Al'tshuler expression, eqn. (6.7)+(6.8) *)
    POW_ETA_Q0:=EXP(Q0[MODE]*ZLN);
    IF ABS(Q0[MODE])<1E-8 THEN Z1:=-GAMMA0[MODE]*ZLN
    ELSE
    BEGIN
      Z1:=-GAMMA_INF[MODE]*ZLN-(GAMMA0[MODE]-GAMMA_INF[MODE])*(POW_ETA_Q0-1)/Q0[MODE]
    END
  END;
  IF GRUNEISEN_TYPE=2 THEN
  BEGIN
    (* Stixrude-Lithgow-Bertelloni (2005) elasticity formalism, eqn. (6.13)+(6.14) *)
    EX:=-2/3; Z:=POW_N(ETA,EX); STRAIN:=(Z-1)/2;
    A1:=6*GAMMA0[MODE];
    A2:=-12*GAMMA0[MODE]+36*GAMMA0[MODE]*GAMMA0[MODE]-18*GAMMA0[MODE]*Q0[MODE];
    (* Note that the argument of the LN can be negative when strain is negative.
       In that case we have imaginary frequency in eqn. (3.7) of Gruneis.doc. It
       occurs when volume is very large, far beyond melting and happens normally
       when V->infinity calculations.
       We set in that case that the argument LN_ARG is 1e-08 to prevent a
       runtime error.
       Eqn. (6.9)
     *)
    LN_ARG:=1+A1*STRAIN+0.5*A2*STRAIN*STRAIN;
    IF LN_ARG<1E-8 THEN LN_ARG:=1E-08;
    Z1:=0.5*LN(LN_ARG)
  END;
  (* Final result of the frequency. Argument, Z1, of exp must be larger than
     -max_arg_exp=-11200
   *)
  IF Z1<-MAX_ARG_EXP THEN Z1:=-MAX_ARG_EXP;
  THETA:=THETA0[MODE]*EXP(Z1);
  (* Mode Gruneisen parameter *)
  IF GRUNEISEN_TYPE=1 THEN
  BEGIN
    (* Gruneisen expression of Al'tshuler, eqn. (6.4) *)
    GAMMA:=GAMMA_INF[MODE]+(GAMMA0[MODE]-GAMMA_INF[MODE])*POW_ETA_Q0
  END;
  IF GRUNEISEN_TYPE=2 THEN
  BEGIN
    (* Gruneisen expression used by Stixrude-Lithgow-Bertelloni (2005).
       Variables strain, a1, a2 were defined in the calculation of frequency.
       Eqn. (6.11)
     *)
    Z:=THETA0[MODE]/THETA; Z:=Z*Z;
    GAMMA:=Z*(2*STRAIN+1)*(A1+A2*STRAIN)/6
  END
END;

PROCEDURE INIT_THETA_GAMMA(VPT:EXTENDED);
(* Initializes X=h*nu/k=theta and Gruneisen parameter, Gamma for use in
   THERMAL_PRESSURE and SINGLE_VIBRATION_PROPERTY. That is done for all modes.
   Theta and Gamma in the resulting arrays THETA_TV and GAMMA_TV depend on
   volume only
 *)
VAR ETA,GAMMA,XV:EXTENDED;
    J:INTEGER;
BEGIN
  ETA:=VPT/V0;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    (* At volume V store Gruneisen parameters and frequencies in GAMMA_TV,
       THETA_TV. Note that Gruneisen and frequency are strictly volume
       dependent and not temperature dependent. No effect of anharmonicity
       should be included: that is done independently via perturbation
       theory
     *)
    MAKE_GRUN_THETA(ETA,J,GAMMA,XV);
    GAMMA_TV[J]:=GAMMA; THETA_TV[J]:=XV;
  END
END;

FUNCTION THERMAL_PRESSURE(VPT:EXTENDED):EXTENDED;
(* Calculation of thermal pressure at volume VPT and temperature TEMPERATURE.
   TEMPERATURE is a global variable
 *)
VAR J:INTEGER;
    GAMMA,X,P_VIB,FAC,Z,Z_EXP,A_AN,DA_AN:EXTENDED;
BEGIN
  (* Calculate all Einstein temperatures and Gruneisen parameters in VDoS *)
  INIT_THETA_GAMMA(VPT);
  P_VIB:=0;
  (* Factors 3nRT/V (T>0) and 3nR/V (T=0) *)
  IF TEMPERATURE>1E-12 THEN FAC:=3*N_MOL*GAS_CONSTANT*TEMPERATURE/VPT
  ELSE FAC:=3*N_MOL*GAS_CONSTANT/VPT;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN    
    IF FRAC[J]>1E-13 THEN
    BEGIN
      IF TEMPERATURE>1E-12 THEN X:=THETA_TV[J]/TEMPERATURE
      ELSE X:=THETA_TV[J];
      GAMMA:=GAMMA_TV[J];
      IF TEMPERATURE>1E-12 THEN
      BEGIN
        (* Prevent negative frequencies *)
        IF X<=0 THEN X:=CRIT_ARG_EXP;
        IF X>MAX_ARG_EXP THEN Z_EXP:=EXP(MAX_ARG_EXP) ELSE Z_EXP:=EXP(X);
        (* Quasi-harmonic: eqn. (6.25a), 1st term *)
        IF X>CRIT_ARG_EXP THEN
        BEGIN
          IF X>MAX_ARG_EXP THEN Z:=X/2 ELSE Z:=X/2+X/(Z_EXP-1)
        END
        ELSE Z:=X/2+1;
        (* Anharmonic contribution to pressure, eqn. (6.25a), 2nd term *)
        Z:=Z*GAMMA+P_ANHARMONIC(J,GAMMA,VPT,X);
      END
      ELSE
      BEGIN
        (* Calculate properties at zero Kelvin: Quasi-harmonic, eqn. (6.25b) *)
        Z:=X/2;
        (* Anharmonic contribution at zero Kelvin, eqn. (6.25d) *)
        Z:=Z*GAMMA;
        A_AN:=ANH_A(J,0,VPT); DA_AN:=ANH_A(J,1,VPT);
        Z:=Z+X*X*(A_AN*GAMMA/12-DA_AN*VPT/24)
      END;
      Z:=Z*FAC
    END
    ELSE Z:=0;
    Z:=Z*FRAC[J];
    P_VIB:=P_VIB+Z
  END;
  THERMAL_PRESSURE:=P_VIB
END;

FUNCTION MAKE_DGAMMA_DV(J:INTEGER;ETA:EXTENDED):EXTENDED;
(* Makes the function -V*(dGamma/dV)T needed in isothermal bulk modulus.
   J denotes the j-th Einstein temperature. ETA is compression V/V0.
   Eqn. (6.6) + (6.12)
 *)
VAR EX,Z1,Z2,HULP,ZV,A1_S,A2_S,STRAIN:EXTENDED;
BEGIN
  ZV:=LN(ETA);
  IF GRUNEISEN_TYPE=1 THEN
  BEGIN
    (* Gruneisen expression of Al'tshuler, eqn. (6.6) *)
    MAKE_DGAMMA_DV:=-Q0[J]*(GAMMA0[J]-GAMMA_INF[J])*EXP(Q0[J]*ZV)
  END;
  IF GRUNEISEN_TYPE=2 THEN
  BEGIN
    (* Gruneisen expression used by Stixrude-Lithgow-Bertelloni (2005).
       Arrays for frequency THETA_TV and Gruneisen GAMMA_TV were prepared in
       INIT_THETA_GAMMA. Eqn. (6.12)
     *)
    A1_S:=6*GAMMA0[J];
    A2_S:=-12*GAMMA0[J]+36*GAMMA0[J]*GAMMA0[J];
    A2_S:=A2_S-18*GAMMA0[J]*Q0[J];
    HULP:=THETA0[J]/THETA_TV[J]; HULP:=HULP*HULP;
    EX:=-2/3; Z2:=POW_N(ETA,EX); STRAIN:=(Z2-1)/2; 
    HULP:=-0.5*HULP*(2*STRAIN+1)*(2*STRAIN+1)*A2_S;
    Z1:=-(18*GAMMA_TV[J]*GAMMA_TV[J]-6*GAMMA_TV[J]+HULP)/9;
    MAKE_DGAMMA_DV:=Z1
  END
END;

FUNCTION SINGLE_VIBRATION_PROPERTY(WHAT:INTEGER;VPT,T_IN:EXTENDED):EXTENDED;
(* Vibrational contribution of a single thermodynamic property.
   VPT is the volume and T_IN the temperature. 
   WHAT: 1=energy, 2=entropy, 3=Cv, 4=dP/dT)V=isochoric temperature derivative
         of thermal pressure, 5=Helmholtz energy, 6=isothermal bulk modulus,
         7=zeropoint energy.
   If T_IN<1e-12 then properties at zero Kelvin are calculated.
   Documents: me-freepascal-manual.doc
 *)
VAR J:INTEGER;
    X,PVT,FAC,Z,Z1,Z2,Z_EXP,ETA,HULP,GAMMA,A_PAR,A1_PAR,ZTN,A2_PAR,MULT,
    LN_HELM,DGAMDV:EXTENDED;
BEGIN
  (* Calculation of the property at T,V *)
  PVT:=0; ETA:=VPT/V0;
  (* Preparing factors *)
  FAC:=3*N_MOL*GAS_CONSTANT;
  IF (WHAT=1) OR (WHAT>4) THEN
  BEGIN
    (* Factor for energy, helmholtz, bulkmodulus, zeropoint energy *)
    IF T_IN>1E-12 THEN FAC:=FAC*T_IN
  END;
  (* Factor for dP/dT)V, bulkmodulus *)
  IF (WHAT=4) OR (WHAT=6) THEN FAC:=FAC/VPT;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    (* Fraction of frequency in VDoS *)
    MULT:=FRAC[J];
    IF MULT>1E-13 THEN
    BEGIN
      IF T_IN>1E-12 THEN X:=THETA_TV[J]/T_IN ELSE X:=THETA_TV[J];
      GAMMA:=GAMMA_TV[J];
      (* Prevent negative X *)
      IF X<=0 THEN X:=CRIT_ARG_EXP;
      IF X>MAX_ARG_EXP THEN Z_EXP:=EXP(MAX_ARG_EXP) ELSE Z_EXP:=EXP(X);
      (* We must restrict the value for exp(x) because powers are taken and the limit
         for extended real is 1e+4950
       *)
      IF Z_EXP>RESTRICT_EXP THEN Z_EXP:=RESTRICT_EXP;
      (* For small X the logarithm in the QHM Helmholtz energy and entropy becomes unstable
         therefore it is approximated as 1/Z_EXP=exp(-x)=1-x and Ln(1-exp(-x))=ln(x)
       *)
      IF X>CRIT_ARG_EXP THEN LN_HELM:=LN(1-1/Z_EXP) ELSE LN_HELM:=LN(X);
      CASE WHAT OF
        1:BEGIN
            (* Energy, eqn. (6.23) *)
            IF T_IN>1E-12 THEN
            BEGIN
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                (* Quasi-harmonic part, 1st term (6.23) *)
                IF X<MAX_ARG_EXP THEN HULP:=X/2+X/(Z_EXP-1)
                ELSE HULP:=X/2
              END
              ELSE HULP:=1;
              (* Adding anharmonic part, 2nd term (6.23) *)
              HULP:=HULP+U_ANHARMONIC(J,T_IN,VPT,X)
            END
            ELSE
            BEGIN
              (* At T=0: Quasi-harmonic (6.23b) *)
              HULP:=X/2;
              A_PAR:=ANH_A(J,0,VPT);
              (* At T=0: Anharmonic (6.23d) *)
              HULP:=HULP+X*X*A_PAR/24
            END;
            PVT:=PVT+FAC*MULT*HULP
          END;
        2:BEGIN
            (* Entropy, eqn. (6.22a) *)
            IF T_IN>1E-12 THEN
            BEGIN
              (* Anharmonic term (6.22a) 2nd term *)
              A_PAR:=ANH_A(J,0,VPT);
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN
                BEGIN
                  HULP:=A_PAR*T_IN*X*X*X*Z_EXP*(Z_EXP+1)/(Z_EXP-1)/(Z_EXP-1)/(Z_EXP-1)/2
                END
                ELSE HULP:=0
              END
              ELSE HULP:=A_PAR*T_IN;
              (* Quasi-harmonic part (6.22a), 1st term *)
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN PVT:=PVT+FAC*MULT*(X/(Z_EXP-1)-LN_HELM-HULP)
                ELSE PVT:=PVT+0
              END
              ELSE PVT:=PVT+FAC*MULT*(1-LN_HELM-HULP)
            END
            ELSE PVT:=0
          END;
        3:BEGIN
            (* Isochoric heat capacity, eqn. (6.24a) *)
            IF T_IN>1E-12 THEN
            BEGIN
              (* Quasi-harmonic part, eqn. (6.24a) 1st term *)
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN Z1:=X*X*Z_EXP/(Z_EXP-1)/(Z_EXP-1) ELSE Z1:=0
              END
              ELSE Z1:=1;
              (* Anharmonic part, eqn. (6.24a) 2nd term *)
              Z1:=Z1+CV_ANHARMONIC(J,T_IN,VPT,X);
              PVT:=PVT+FAC*MULT*Z1
            END
            ELSE PVT:=0
          END;
        4:BEGIN
            (* (dP/dT)V, eqn. (6.26a) *)
            IF T_IN>1E-12 THEN
            BEGIN
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN
                BEGIN
                  (* Quasi-harmonic, eqn. (6.26a) 1st term: normal values for X *)
                  Z1:=X/(Z_EXP-1); Z1:=Z1*Z1*Z_EXP
                END
                ELSE Z1:=0
              END
              ELSE Z1:=1;
              (* Adding anharmonic part, eqn. (6.26a) 2nd term *)
              Z1:=Z1*GAMMA+DP_ANHARMONIC(J,T_IN,GAMMA,VPT,X);
              PVT:=PVT+FAC*MULT*Z1
            END
            ELSE PVT:=0
          END;
        5:BEGIN
            (* Helmholtz energy, eqn. (6.21a) *)
            IF T_IN>1E-12 THEN
            BEGIN
              (* Quasi-harmonic part, eqn. (6.21a), 1st term *)
              Z1:=LN_HELM+X/2;
              (* Anharmonic part, eqn. (6.21a) 2nd term *)
              A_PAR:=ANH_A(J,0,VPT);
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN HULP:=X*X/4+3*X*X*Z_EXP/(Z_EXP-1)/(Z_EXP-1)
                ELSE HULP:=X*X/4
              END
              ELSE HULP:=3;
              Z1:=Z1+HULP*A_PAR*T_IN/6;
              PVT:=PVT+FAC*MULT*Z1
            END
            ELSE
            BEGIN
              (* At T=0: Quasi-harmonic, eqn. (6.21b) *)
              PVT:=PVT+FAC*MULT*X/2;
              (* At T=0: Anharmonic, eqn. (6.21d) *)
              A_PAR:=ANH_A(J,0,VPT);
              PVT:=PVT+FAC*MULT*X*X*A_PAR/24
            END
          END;
        6:BEGIN
            (* Isothermal bulk modulus, eqn. (6.27a). Construct for mode j: -V*(dGamma)/dV) *)
            Z1:=MAKE_DGAMMA_DV(J,ETA);
            DGAMDV:=Z1;
            IF T_IN>1E-12 THEN
            BEGIN
              (* Quasi-harmonic *)
              Z2:=Z1+GAMMA*GAMMA+GAMMA;
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN
                BEGIN
                  (* Not extreme values of X, eqn. (6.27a) *)
                  Z1:=-GAMMA*GAMMA*X*X*Z_EXP/(Z_EXP-1)/(Z_EXP-1);
                  Z1:=Z1+Z2*(X/(Z_EXP-1)+X/2)
                END
                ELSE
                BEGIN
                  (* Extremely large value for X, eqn. (6.27b) *)
                  Z1:=Z2*X/2
                END
              END
              ELSE
              BEGIN
                (* Extremely small values for X: when X=0 the equation below
                   converges to (6.27c)
                 *)
                Z1:=-GAMMA*GAMMA; Z1:=Z1+Z2*(1+X/2)
              END;
              (* Anharmonic perturbation theory *)
              Z1:=Z1+K_ANHARMONIC(J,T_IN,GAMMA,DGAMDV,VPT,X)
            END
            ELSE
            BEGIN
              (* At T=0: Quasi-harmonic, eqn. (6.27b) *)
              Z1:=(GAMMA*GAMMA+GAMMA+Z1)*X/2;
              (* Anharmonic perturbation theory, eqn. (6.27d) *)
              A_PAR:=ANH_A(J,0,VPT);
              A1_PAR:=ANH_A(J,1,VPT);
              A2_PAR:=ANH_A(J,2,VPT);
              HULP:=X*X;
              ZTN:=A_PAR*(GAMMA+DGAMDV)/12+A_PAR*GAMMA*GAMMA/6;
              ZTN:=ZTN+A2_PAR*VPT*VPT/24-GAMMA*A1_PAR*VPT/12;
              Z1:=Z1+ZTN*HULP
            END;
            Z1:=Z1*FAC*MULT;
            PVT:=PVT+Z1
          END;
        7:BEGIN
            (* Zero point energy: energy at 0 Kelvin, eqn. (6.23b,d) *)
            HULP:=X/2;
            (* Anharmonic perturbation theory: no multply with T, because it factors
               out by definition of X at T=0 above
             *)
            A_PAR:=ANH_A(J,0,VPT); Z1:=X*X/24;
            HULP:=HULP+A_PAR*Z1;
            PVT:=PVT+FAC*MULT*HULP
          END
      END
    END
  END;
  SINGLE_VIBRATION_PROPERTY:=PVT
END;

PROCEDURE VIBRATION_PROPERTIES(VPT:EXTENDED;VAR ENERGY,ENTROPY,CV,DPDT,
                               HELMHOLTZ,BULKMOD,ZERO_PNT:EXTENDED);
(* Main procedure to calculate vibrational properties. Volume is VPT. CV is
   isochoric heat capacity, BULKMOD isothermal bulk modulus, DPDT is the
   isochoric change of thermal pressure with temperature. ZERO_PNT is
   zeropoint pressure: thermal pressure at 0 Kelvin, 0 Pascal
 *)
BEGIN
  INIT_THETA_GAMMA(VPT);
  (* Helmholtz energy *)
  HELMHOLTZ:=SINGLE_VIBRATION_PROPERTY(5,VPT,TEMPERATURE);
  (* Entropy *)
  ENTROPY:=SINGLE_VIBRATION_PROPERTY(2,VPT,TEMPERATURE);
  (* Energy: calculate it from Helmholtz energy and entropy *)
  ENERGY:=HELMHOLTZ+TEMPERATURE*ENTROPY;
  (* Isochoric heat capacity *)
  CV:=SINGLE_VIBRATION_PROPERTY(3,VPT,TEMPERATURE);
  (* Isochoric change of thermal pressure with temperature *)
  DPDT:=SINGLE_VIBRATION_PROPERTY(4,VPT,TEMPERATURE);
  (* Isothermal bulk modulus *)
  BULKMOD:=SINGLE_VIBRATION_PROPERTY(6,VPT,TEMPERATURE);
  (* Zero point energy contribution *)
  ZERO_PNT:=SINGLE_VIBRATION_PROPERTY(7,VPT,TEMPERATURE)
END;

FUNCTION U_NS_VIB(WHAT:INTEGER;VPT,T_IN:EXTENDED):EXTENDED;
(* Vibrational part of shear modulus, eqn. (6.15).
   Vibrational Energy, and zeropoint energy weighed to nsj
   array based on combining the multiple-Einstein method and elasticity of
   Stixrude-Lithgow-Bertelloni (2005).
   WHAT: 1=energy, (2=entropy, 5=Helmholtz not used), 7=zeropoint energy.
   If T<1e-12 then properties at zero Kelvin.
 *)
VAR J:INTEGER;
    X,PVT,FAC,Z_EXP,HULP,A_PAR,MULT:EXTENDED;
BEGIN
  (* The calculation of the property at T,V *)
  PVT:=0;
  FAC:=3*N_MOL*GAS_CONSTANT;
  IF (WHAT=1) OR (WHAT>4) THEN
  BEGIN
    IF T_IN>1E-12 THEN FAC:=FAC*T_IN
  END;
  IF (WHAT=4) OR (WHAT=6) THEN FAC:=FAC/VPT;
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    MULT:=FRAC[J]*NS_STIX[J];
    IF MULT>1E-13 THEN
    BEGIN
      IF T_IN>1E-12 THEN X:=THETA_TV[J]/T_IN ELSE X:=THETA_TV[J];
      (* No negative frequencies allowed *)
      IF X<=0 THEN X:=CRIT_ARG_EXP;
      IF X<MAX_ARG_EXP THEN Z_EXP:=EXP(X) ELSE Z_EXP:=EXP(MAX_ARG_EXP);
      (* We must restrict the value for exp(x) because powers are taken and the limit for
         extended real is 1e+4950
       *)
      IF Z_EXP>RESTRICT_EXP THEN Z_EXP:=RESTRICT_EXP;
      CASE WHAT OF
        1:BEGIN
            (* Energy, eqn. (6.23a) *)
            IF T_IN>1E-12 THEN
            BEGIN
              IF X>CRIT_ARG_EXP THEN
              BEGIN
                IF X<MAX_ARG_EXP THEN
                BEGIN
                  (* Not extreme values for X, Quasi-harmonic *)
                  HULP:=X/2+X/(Z_EXP-1)
                END
                ELSE
                BEGIN
                  (* Extreme values for X, eqn. (6.23b) *)
                  HULP:=X/2
                END
              END
              ELSE HULP:=1;
              (* Anharmonic part *)
              HULP:=HULP+U_ANHARMONIC(J,T_IN,VPT,X)
            END
            ELSE
            BEGIN
              (* At T=0: Quasi-harmonic, eqn. (6.23b) *)
              HULP:=X/2;
              (* At T=0: Anharmonic term, eqn. (6.23d) *)
              A_PAR:=ANH_A(J,0,VPT);
              HULP:=HULP+X*X*A_PAR/24
            END;
            PVT:=PVT+FAC*MULT*HULP
          END;
        7:BEGIN
            (* At T=0: Zeropoint energy T=0, eqn. (6.23b) *)
            HULP:=X/2;
            (* At T=0: Anharmonic term, eqn. (6.23d) *)
            A_PAR:=ANH_A(J,0,VPT);
            HULP:=HULP+X*X*A_PAR/24;
            PVT:=PVT+FAC*MULT*HULP
          END
      END
    END
  END;
  U_NS_VIB:=PVT
END;

PROCEDURE SHEAR_VIB(VPT:EXTENDED;VAR GSH_VIB:EXTENDED);
(* Main procedure to calculate vibrational shear modulus based on the formalism
   of Stixrude-Lithgow-Bertelloni (2005), eqn. (6.15) + (6.16)
 *)
VAR J:INTEGER;
    ETA,EX,STRAIN,U,Z:EXTENDED;
BEGIN
  (* Shear modulus model is based on finite strain. The static equation of state
     must be based on Birch_Murnaghan: EOS_TYPE=2
   *)
  IF EOS_TYPE=2 THEN
  BEGIN
    (* Determine Einstein temperature, Gruneisen at (T,V) *)
    INIT_THETA_GAMMA(VPT);
    (* Make the ns(j) array with j the mode. Note that we have to use the quasi-harmonic
       frequencies just as in any thermodynamic property when perturbation theory.
       Eqn. (6.16)
     *)
    ETA:=VPT/V0;
    EX:=-2/3; STRAIN:=(POW_N(ETA,EX)-1)/2;
    FOR J:=1 TO N_EINSTEIN DO
    BEGIN
      Z:=THETA0[J]*(2*STRAIN+1)/THETA_TV[J];
      NS_STIX[J]:=-GAMMA_TV[J]+(GAMMA0[J]+NS0)*Z*Z;
    END;
    (* Calculate the vibrational energy.
       If T=0 calculate the zeropoint energy. Note that although NS0 is a constant
       NS_STIX may be disperse due to dispersion in Gruneisen parameters. 
       Gsh_vib=-(1/V)Sum(nsj*Uj) when summing over modes j
     *)
    IF TEMPERATURE>1E-12 THEN
    BEGIN
      U:=U_NS_VIB(1,VPT,TEMPERATURE)
    END
    ELSE U:=U_NS_VIB(7,VPT,TEMPERATURE);
    GSH_VIB:=-U/VPT
  END
  ELSE
  BEGIN
    (* Shear modulus has not been programmed for EoS that is not based
       on finite strain, therefore it is set to zero
     *)
    GSH_VIB:=0
  END
END;

END.
